use Test::More;
use strict;

use DE_EPAGES::Core::API::PerlTools qw (fcmp);
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore GetTestSite  );
use DE_EPAGES::Object::API::Factory qw ( ExistsObjectByPath LoadRootObject LoadObjectByPath LoadClassByAlias );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::Amazon::API::DTO::OrderReport;
use DE_EPAGES::Amazon::API::DTO::Address;
use DE_EPAGES::Amazon::API::AmazonTransaction;
use DE_EPAGES::Amazon::API::DTO::Component;
use DE_EPAGES::Amazon::API::DTO::Item;

my $ROOTPATH = '/Test';

sub getTestData {
    my $O = DE_EPAGES::Amazon::API::DTO::OrderReport->new;
    $O->AmazonOrderID('302-6636987-0853930');
    $O->AmazonSessionID('302-6636987-0853930');
    $O->OrderDate('2009-06-30T13:17:49+00:00');
    $O->ReportNumber('922629424');

    $O->BuyerEmailAddress('buyer@example.com');
    $O->BuyerName('Hansi Müller');

    $O->Currency('EUR');
    $O->FulfillmentAddress('Lieferstr. 553');
    $O->FulfillmentMethod('Ship');
    $O->FulfillmentServiceLevel('Standard');

    my $B = DE_EPAGES::Amazon::API::DTO::Address->new;
    $B->Name('Hansi Müller');
    $B->AddressFieldOne('Kaeufergasse 123');
    $B->AddressFieldTwo('');
    $B->City('Hamburg');
    $B->StateOrRegion('Hamburg');
    $B->PostalCode('20222');
    $B->CountryCode('DE');
    $B->PhoneNumber('+49 40 123 123432');

    $O->BuyerAddress($B);

    my $F = DE_EPAGES::Amazon::API::DTO::Address->new;
    $F->Name('Petra Zwegert');
    $F->AddressFieldOne('Lieferstr. 14');
    $F->AddressFieldTwo('c/o Hermine Meier');
    $F->City('Hamburg');
    $F->StateOrRegion('Hamburg');
    $F->PostalCode('20345');
    $F->CountryCode('DE');
    $F->PhoneNumber('+49 40 123 123433');

    $O->FulfillmentAddress($F);

    my $I = DE_EPAGES::Amazon::API::DTO::Item->new;
    $I->AmazonOrderItemCode('05991383163091');
    $I->ProductTaxCode('PTC_PRODUCT_TAXABLE_A');
    $I->Title('Campingaz CV470 Ventilgaskartusche');
    $I->Quantity('2');
    $I->SKU('cg_0101104470');

    my $P = DE_EPAGES::Amazon::API::DTO::Component->new;
    $P->Amount(1.55);
    $P->Type('Principal');

    my $P2 = DE_EPAGES::Amazon::API::DTO::Component->new;
    $P2->Amount(3.55);
    $P2->Type('Shipping');

    $I->ItemPriceArray($P);
    $I->ItemPriceArray($P2);

    $O->ItemArray($I);

    return $O;
}

sub testInsert {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            ok( Init(), 'initialization' );

            my $Shop       = LoadObjectByPath( $ROOTPATH . '/TestShopAmazon' );
            my $Merchant   = $Shop->folder('PortalConfigs')->child('AmazonTestConfig');

            my $AmazonTransaction = DE_EPAGES::Amazon::API::AmazonTransaction->new;
            my $testData          = getTestData;

            # prepare stock test
            my $Product = $Shop->folder('Products')->child('cg_0101104470');
            my $OldStock = $Product->get("StockLevel");

            my $dbtlist = LoadClassByAlias('AmazonTransaction')
              ->getObjects( 'ByExternalOrderID', [ $testData->AmazonOrderID ] );
            ## delete existing transations so that the import will work.
            foreach my $dbt (@$dbtlist) {
                $dbt->delete;
            }

            my @orderids = $AmazonTransaction->insert( $Merchant, [$testData] );
            ok( scalar @orderids == 1, "1 order generated." );
            my $orderid = pop @orderids;
            $dbtlist = LoadClassByAlias('AmazonTransaction')
              ->getObjects( 'ByExternalOrderID', [$orderid] );

            ok( scalar @$dbtlist >= 1, 'at least 1 transaction generated' );
            my $dbt = pop @$dbtlist;

            # search order..
            my $matchingLineItems = $AmazonTransaction->getImportedLineItems();
            ok( scalar @$matchingLineItems >= 1, 'at least 1 matching lineitem generated' );
            my $lineItem = pop @$matchingLineItems;

            my $NewStock;
            $NewStock = $Product->get("StockLevel");
            ok( $NewStock == ( $OldStock - 2 ),
                "stock before import: " . $OldStock . ' - after import ' . $NewStock );

            my $dbcontainer = $lineItem->get('LineItemContainer');
            my $dborder     = $dbcontainer->parent;

            # check lineitempayment amount
            if ( !$dbcontainer->get("Payment")->class->existsAttribute('Amount') ) {
                ok( 1, 'regular payment.' );
            }
            else {
                ok(
                    fcmp( $dbcontainer->get("Payment")->get("Amount"),
                        $dbcontainer->get("GrandTotal") ) == 0,
                    "LineItemPayment amount"
                );
            }

            # check billing address
            my $dbbillingAddress   = $dborder->get('BillingAddress');
            my $testbillingAddress = $testData->BuyerAddress;
            ok( $dbbillingAddress->get('City') eq $testbillingAddress->City, 'billing city' );
            ok( $dbbillingAddress->get('Street') eq $testbillingAddress->AddressFieldOne,
                'billing street' );
            ok( $dbbillingAddress->get('State') eq $testbillingAddress->StateOrRegion,
                'billing state' );
            ok( $dbbillingAddress->get('Street2') eq $testbillingAddress->AddressFieldTwo,
                'billing street 2' );
            ok( $dbbillingAddress->get('Zipcode') eq $testbillingAddress->PostalCode,
                'billing plz' );
            ok( $dbbillingAddress->get('EMail') eq $testData->BuyerEmailAddress, 'billing email' );

            # check fulfillment address
            my $dbfulfillmentAddress   = $dborder->get('ShippingAddress');
            my $testfulfillmentAddress = $testData->FulfillmentAddress;

            ok( $dbfulfillmentAddress->get('City') eq $testfulfillmentAddress->City,
                'shipping city' );
            ok( $dbfulfillmentAddress->get('Street') eq $testfulfillmentAddress->AddressFieldOne,
                'shipping street' );
            ok( $dbfulfillmentAddress->get('State') eq $testfulfillmentAddress->StateOrRegion,
                'shipping state' );
            ok( $dbfulfillmentAddress->get('Street2') eq $testfulfillmentAddress->AddressFieldTwo,
                'shipping street 2' );
            ok( $dbfulfillmentAddress->get('Zipcode') eq $testfulfillmentAddress->PostalCode,
                'shipping plz' );

            # test product delete
            $Product->delete;
            pass('product delete trigger test.');

            # cleanup
            ok( Cleanup(), 'cleanup');
        }
    );
}

sub Main {
    testInsert;
    done_testing;
    return;
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    my $Folder = LoadRootObject()->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects('import.xml', $ROOTPATH);
    return 1;
}

RunScript( Sub => \&Main );

1;
