use Test::Simple tests => 308;
use strict;

use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject ExistsObjectByPath LoadObjectByPath LoadClassByAlias);
use DE_EPAGES::XML::API::DateTimeFormatXML;

my $Parser = DE_EPAGES::XML::API::DateTimeFormatXML->new();
my $ROOTPATH = '/Test';

my @TESTS = (
    {   'name'  => 'f1f1_11',
        'locks' => [ ['10:00', '11:00', 1], ['09:00', '09:30', 1] ],
        'count' => 2,
        'tests' => {'08:30'=>0, '09:00'=>1,'09:30'=>0, '09:45'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>0,'11:30'=>0 }
    },
    {   'name'  => 'f1s1_11', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['09:00', '10:00', 1] ],
        'count' => 1,
        'tests' => {'08:30'=>0, '09:00'=>1,'09:30'=>1, '09:45'=>1, '10:00'=>1,'10:30'=>1,'11:00'=>0,'11:30'=>0 }
    },
    {   'name'  => 'f1m1_11', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['09:00', '10:30', 1] ],
        'count' => 1,
        'tests' => {'08:30'=>0, '09:00'=>1,'09:30'=>1, '09:45'=>1, '10:00'=>1,'10:30'=>1,'11:00'=>0,'11:30'=>0 }
    },
    {   'name'  => 'f1e1_11', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['09:00', '11:00', 1] ],
        'count' => 1,
        'tests' => {'08:30'=>0, '09:00'=>1,'09:30'=>1, '09:45'=>1, '10:00'=>1,'10:30'=>1,'11:00'=>0,'11:30'=>0 }
    },
    {   'name'  => 'f1b1_11', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['09:00', '11:30', 1] ],
        'count' => 1,
        'tests' => {'08:30'=>0, '09:00'=>1,'09:30'=>1, '09:45'=>1, '10:00'=>1,'10:30'=>1,'11:00'=>1,'11:30'=>0 }
    },
    {   'name'  => 's1m1_11', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['10:00', '10:30', 1] ],
        'count' => 1,
        'tests' => {'08:30'=>0, '09:00'=>0,'09:30'=>0, '09:45'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>0,'11:30'=>0 }
    },
    {   'name'  => 's1e1_11', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['10:00', '11:00', 1] ],
        'count' => 1,
        'tests' => {'08:30'=>0, '09:00'=>0,'09:30'=>0, '09:45'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>0,'11:30'=>0 }
    },
    {   'name'  => 's1b1_11', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['10:00', '11:30', 1] ],
        'count' => 1,
        'tests' => {'08:30'=>0, '09:00'=>0,'09:30'=>0, '09:45'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>1,'11:30'=>0 }
    },
    {   'name'  => 'm1m1_11', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['10:30', '10:45', 1] ],
        'count' => 1,
        'tests' => {'08:30'=>0, '09:00'=>0,'09:30'=>0, '09:45'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>0,'11:30'=>0 }
    },
    {   'name'  => 'm1e1_11', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['10:30', '11:00', 1] ],
        'count' => 1,
        'tests' => {'08:30'=>0, '09:00'=>0,'09:30'=>0, '09:45'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>0,'11:30'=>0 }
    },
    {   'name'  => 'm1b1_11', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['10:30', '11:30', 1] ],
        'count' => 1,
        'tests' => {'08:30'=>0, '09:00'=>0,'09:30'=>0, '09:45'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>1,'11:30'=>0 }
    },
    {   'name'  => 'e1b1_11', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['11:00', '11:30', 1] ],
        'count' => 1,
        'tests' => {'08:30'=>0, '09:00'=>0,'09:30'=>0, '09:45'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>1,'11:30'=>0 }
    },
    {   'name'  => 'b1b1_11', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['11:30', '12:00', 1] ],
        'count' => 2,
        'tests' => {'08:30'=>0, '09:00'=>0,'09:30'=>0, '09:45'=>0, '10:00'=>1,'10:30'=>1,'11:15'=>0,'11:30'=>1 }
    },
    {   'name'  => 'f1f1_01',
        'locks' => [ ['10:00', '11:00', 0], ['09:00', '09:30', 1] ],
        'count' => 2,
        'tests' => {'08:30'=>0, '09:00'=>1,'09:30'=>0, '09:45'=>0, '10:00'=>0,'10:30'=>0,'11:00'=>0,'11:30'=>0 }
    },
    {   'name'  => 'f1s1_01', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['09:00', '10:00', 1] ],
        'count' => 2,
        'tests' => {'08:30'=>0, '09:00'=>1,'09:30'=>1, '09:45'=>1, '10:00'=>0,'10:30'=>0,'11:00'=>0,'11:30'=>0 }
    },
    {   'name'  => 'f1m1_01', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['09:00', '10:30', 1] ],
        'count' => 2,
        'tests' => {'08:30'=>0, '09:00'=>1,'09:30'=>1, '09:45'=>1, '10:00'=>1,'10:30'=>0,'11:00'=>0,'11:30'=>0 }
    },
    {   'name'  => 'f1e1_01', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['09:00', '11:00', 1] ],
        'count' => 1,
        'tests' => {'08:30'=>0, '09:00'=>1,'09:30'=>1, '09:45'=>1, '10:00'=>1,'10:30'=>1,'11:00'=>0,'11:30'=>0 }
    },
    {   'name'  => 'f1b1_01', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['09:00', '11:30', 1] ],
        'count' => 1,
        'tests' => {'08:30'=>0, '09:00'=>1,'09:30'=>1, '09:45'=>1, '10:00'=>1,'10:30'=>1,'11:00'=>1,'11:30'=>0 }
    },
    {   'name'  => 's1m1_01', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['10:00', '10:30', 1] ],
        'count' => 2,
        'tests' => {'08:30'=>0, '09:00'=>0,'09:30'=>0, '09:45'=>0, '10:00'=>1,'10:30'=>0,'11:00'=>0,'11:30'=>0 }
    },
    {   'name'  => 's1e1_01', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['10:00', '11:00', 1] ],
        'count' => 1,
        'tests' => {'08:30'=>0, '09:00'=>0,'09:30'=>0, '09:45'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>0,'11:30'=>0 }
    },
    {   'name'  => 's1b1_01', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['10:00', '11:30', 1] ],
        'count' => 1,
        'tests' => {'08:30'=>0, '09:00'=>0,'09:30'=>0, '09:45'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>1,'11:30'=>0 }
    },
    {   'name'  => 'm1m1_01', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['10:30', '10:45', 1] ],
        'count' => 3,
        'tests' => {'08:30'=>0, '09:00'=>0,'09:30'=>0, '09:45'=>0, '10:00'=>0,'10:30'=>1,'10:45'=>0,'11:00'=>0 }
    },
    {   'name'  => 'm1e1_01', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['10:30', '11:00', 1] ],
        'count' => 2,
        'tests' => {'08:30'=>0, '09:00'=>0,'09:30'=>0, '09:45'=>0, '10:00'=>0,'10:30'=>1,'11:00'=>0,'11:30'=>0 }
    },
    {   'name'  => 'm1b1_01', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['10:30', '11:30', 1] ],
        'count' => 2,
        'tests' => {'08:30'=>0, '09:00'=>0,'09:30'=>0, '09:45'=>0, '10:00'=>0,'10:30'=>1,'11:00'=>1,'11:30'=>0 }
    },
    {   'name'  => 'e1b1_01', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['11:00', '11:30', 1] ],
        'count' => 2,
        'tests' => {'08:30'=>0, '09:00'=>0,'09:30'=>0, '09:45'=>0, '10:00'=>0,'10:30'=>0,'11:00'=>1,'11:30'=>0 }
    },
    {   'name'  => 'b1b1_01', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['11:30', '12:00', 1] ],
        'count' => 2,
        'tests' => {'08:30'=>0, '09:00'=>0,'09:30'=>0, '09:45'=>0, '10:00'=>0,'10:30'=>0,'11:15'=>0,'11:30'=>1 }
    },
    # 3 elements
    {   'name'  => 'f1f2_111', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['13:00', '14:00', 1], ['09:00', '12:00', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>1,'09:30'=>1, '09:45'=>1, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>0,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'f1s2_111', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['13:00', '14:00', 1], ['09:00', '13:00', 1] ],
        'count' => 1,
        'tests' => {'08:00'=>0, '09:00'=>1,'09:30'=>1, '09:45'=>1, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'f1m2_111', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['13:00', '14:00', 1], ['09:00', '13:30', 1] ],
        'count' => 1,
        'tests' => {'08:00'=>0, '09:00'=>1,'09:30'=>1, '09:45'=>1, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'f1e2_111', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['13:00', '14:00', 1], ['09:00', '14:00', 1] ],
        'count' => 1,
        'tests' => {'08:00'=>0, '09:00'=>1,'09:30'=>1, '09:45'=>1, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'f1b2_111', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['13:00', '14:00', 1], ['09:00', '15:00', 1] ],
        'count' => 1,
        'tests' => {'08:00'=>0, '09:00'=>1,'09:30'=>1, '09:45'=>1, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>1 }
    },
    {   'name'  => 's1f2_111', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['13:00', '14:00', 1], ['10:00', '12:00', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>0,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 's1s2_111', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['13:00', '14:00', 1], ['10:00', '13:00', 1] ],
        'count' => 1,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 's1m2_111', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['13:00', '14:00', 1], ['10:00', '13:30', 1] ],
        'count' => 1,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 's1e2_111', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['13:00', '14:00', 1], ['10:00', '14:00', 1] ],
        'count' => 1,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 's1b2_111', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['13:00', '14:00', 1], ['10:00', '15:00', 1] ],
        'count' => 1,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>1 }
    },
    {   'name'  => 'm1f2_111', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['13:00', '14:00', 1], ['10:30', '12:00', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>0,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'm1s2_111', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['13:00', '14:00', 1], ['10:30', '13:00', 1] ],
        'count' => 1,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'm1m2_111', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['13:00', '14:00', 1], ['10:30', '13:30', 1] ],
        'count' => 1,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'm1e2_111', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['13:00', '14:00', 1], ['10:30', '14:00', 1] ],
        'count' => 1,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'm1b2_111', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['13:00', '14:00', 1], ['10:30', '15:00', 1] ],
        'count' => 1,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>1 }
    },
    {   'name'  => 'e1f2_111', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['13:00', '14:00', 1], ['11:00', '12:00', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>0,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'e1s2_111', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['13:00', '14:00', 1], ['11:00', '13:00', 1] ],
        'count' => 1,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'e1m2_111', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['13:00', '14:00', 1], ['11:00', '13:30', 1] ],
        'count' => 1,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'e1e2_111', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['13:00', '14:00', 1], ['11:00', '14:00', 1] ],
        'count' => 1,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'e1b2_111', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['13:00', '14:00', 1], ['11:00', '15:00', 1] ],
        'count' => 1,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>1 }
    },
    {   'name'  => 'b1f2_111', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['13:00', '14:00', 1], ['12:00', '12:30', 1] ],
        'count' => 3,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>0,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'b1s2_111', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['13:00', '14:00', 1], ['12:00', '13:00', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>0,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'b1m2_111', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['13:00', '14:00', 1], ['12:00', '13:30', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>0,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'b1e2_111', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['13:00', '14:00', 1], ['12:00', '14:00', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>0,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'b1b2_111', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 1], ['13:00', '14:00', 1], ['12:00', '15:00', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>0,'12:00'=>1,'13:00'=>1, '14:00'=>1 }
    },
    {   'name'  => 'f1f2_011', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 1], ['09:00', '12:00', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>1,'09:30'=>1, '09:45'=>1, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>0,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'f1s2_011', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 1], ['09:00', '13:00', 1] ],
        'count' => 1,
        'tests' => {'08:00'=>0, '09:00'=>1,'09:30'=>1, '09:45'=>1, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'f1m2_011', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 1], ['09:00', '13:30', 1] ],
        'count' => 1,
        'tests' => {'08:00'=>0, '09:00'=>1,'09:30'=>1, '09:45'=>1, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'f1e2_011', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 1], ['09:00', '14:00', 1] ],
        'count' => 1,
        'tests' => {'08:00'=>0, '09:00'=>1,'09:30'=>1, '09:45'=>1, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'f1b2_011', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 1], ['09:00', '15:00', 1] ],
        'count' => 1,
        'tests' => {'08:00'=>0, '09:00'=>1,'09:30'=>1, '09:45'=>1, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>1 }
    },
    {   'name'  => 's1f2_011', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 1], ['10:00', '12:00', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>0,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 's1s2_011', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 1], ['10:00', '13:00', 1] ],
        'count' => 1,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 's1m2_011', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 1], ['10:00', '13:30', 1] ],
        'count' => 1,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 's1e2_011', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 1], ['10:00', '14:00', 1] ],
        'count' => 1,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 's1b2_011', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 1], ['10:00', '15:00', 1] ],
        'count' => 1,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>1 }
    },
    {   'name'  => 'm1f2_011', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 1], ['10:30', '12:00', 1] ],
        'count' => 3,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>0,'10:30'=>1,'11:00'=>1,'12:00'=>0,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'mf1s2_011', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 1], ['10:30', '13:00', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>0,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'm1m2_011', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 1], ['10:30', '13:30', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>0,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'm1e2_011', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 1], ['10:30', '14:00', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>0,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'm1b2_011', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 1], ['10:30', '15:00', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>0,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>1 }
    },
    {   'name'  => 'e1f2_011', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 1], ['11:00', '12:00', 1] ],
        'count' => 3,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>0,'10:30'=>0,'11:00'=>1,'12:00'=>0,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'e1s2_011', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 1], ['11:00', '13:00', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>0,'10:30'=>0,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'e1m2_011', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 1], ['11:00', '13:30', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>0,'10:30'=>0,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'e1e2_011', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 1], ['11:00', '14:00', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>0,'10:30'=>0,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'e1b2_011', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 1], ['11:00', '15:00', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>0,'10:30'=>0,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>1 }
    },
    {   'name'  => 'b1f2_011', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 1], ['12:00', '12:30', 1] ],
        'count' => 3,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>0,'10:30'=>0,'11:00'=>0,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'bf1s2_011', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 1], ['12:00', '13:00', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>0,'10:30'=>0,'11:00'=>0,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'b1m2_011', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 1], ['12:00', '13:30', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>0,'10:30'=>0,'11:00'=>0,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'b1e2_011', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 1], ['12:00', '14:00', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>0,'10:30'=>0,'11:00'=>0,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'b1b2_011', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 1], ['12:00', '15:00', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>0,'10:30'=>0,'11:00'=>0,'12:00'=>1,'13:00'=>1, '14:00'=>1 }
    },
    {   'name'  => 'f1f2_001', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 0], ['09:00', '12:00', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>1,'09:30'=>1, '09:45'=>1, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>0,'13:00'=>0, '14:00'=>0 }
    },
    {   'name'  => 'f1s2_001', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 0], ['09:00', '13:00', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>1,'09:30'=>1, '09:45'=>1, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>0, '14:00'=>0 }
    },
    {   'name'  => 'f1m2_001', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 0], ['09:00', '13:30', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>1,'09:30'=>1, '09:45'=>1, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'f1e2_001', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 0], ['09:00', '14:00', 1] ],
        'count' => 1,
        'tests' => {'08:00'=>0, '09:00'=>1,'09:30'=>1, '09:45'=>1, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'f1b2_001', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 0], ['09:00', '15:00', 1] ],
        'count' => 1,
        'tests' => {'08:00'=>0, '09:00'=>1,'09:30'=>1, '09:45'=>1, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>1 }
    },
    {   'name'  => 's1f2_001', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 0], ['10:00', '12:00', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>0,'13:00'=>0, '14:00'=>0 }
    },
    {   'name'  => 's1s2_001', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 0], ['10:00', '13:00', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>0, '14:00'=>0 }
    },
    {   'name'  => 's1m2_001', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 0], ['10:00', '13:30', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 's1e2_001', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 0], ['10:00', '14:00', 1] ],
        'count' => 1,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 's1b2_001', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 0], ['10:00', '15:00', 1] ],
        'count' => 1,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>1,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>1 }
    },
    {   'name'  => 'm1f2_001', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 0], ['10:30', '12:00', 1] ],
        'count' => 3,
        'result'=> [ ['10:00', '10:30', 0], ['10:30', '12:00', 1], ['13:00', '14:00', 0]  ],
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>0,'10:30'=>1,'11:00'=>1,'12:00'=>0,'13:00'=>0, '14:00'=>0 }
    },
    {   'name'  => 'mf1s2_001', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 0], ['10:30', '13:00', 1] ],
        'count' => 3,
        'result'=> [ ['10:00', '10:30', 0], ['10:30', '13:00', 1], ['13:00', '14:00', 0]  ],
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>0,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>0, '14:00'=>0 }
    },
    {   'name'  => 'm1m2_001', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 0], ['10:30', '13:30', 1] ],
        'count' => 3,
        'result'=> [ ['10:00', '10:30', 0], ['10:30', '13:30', 1], ['13:30', '14:00', 0]  ],
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>0,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'm1e2_001', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 0], ['10:30', '14:00', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>0,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'm1b2_001', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 0], ['10:30', '15:00', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>0,'10:30'=>1,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>1 }
    },
    {   'name'  => 'e1f2_001', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 0], ['11:00', '12:00', 1] ],
        'count' => 3,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>0,'10:30'=>0,'11:00'=>1,'12:00'=>0,'13:00'=>0, '14:00'=>0 }
    },
    {   'name'  => 'e1s2_001', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 0], ['11:00', '13:00', 1] ],
        'count' => 3,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>0,'10:30'=>0,'11:00'=>1,'12:00'=>1,'13:00'=>0, '14:00'=>0 }
    },
    {   'name'  => 'e1m2_001', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 0], ['11:00', '13:30', 1] ],
        'count' => 3,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>0,'10:30'=>0,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'e1e2_001', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 0], ['11:00', '14:00', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>0,'10:30'=>0,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'e1b2_001', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 0], ['11:00', '15:00', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>0,'10:30'=>0,'11:00'=>1,'12:00'=>1,'13:00'=>1, '14:00'=>1 }
    },
    {   'name'  => 'b1f2_001', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 0], ['12:00', '12:30', 1] ],
        'count' => 3,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>0,'10:30'=>0,'11:00'=>0,'12:00'=>1,'13:00'=>0, '14:00'=>0 }
    },
    {   'name'  => 'bf1s2_001', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 0], ['12:00', '13:00', 1] ],
        'count' => 3,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>0,'10:30'=>0,'11:00'=>0,'12:00'=>1,'13:00'=>0, '14:00'=>0 }
    },
    {   'name'  => 'b1m2_001', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 0], ['12:00', '13:30', 1] ],
        'count' => 3,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>0,'10:30'=>0,'11:00'=>0,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'b1e2_001', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 0], ['12:00', '14:00', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>0,'10:30'=>0,'11:00'=>0,'12:00'=>1,'13:00'=>1, '14:00'=>0 }
    },
    {   'name'  => 'b1b2_001', # f=vor/before, s=connected start, m=middle, e=connected end, b=nach/behind
        'locks' => [ ['10:00', '11:00', 0], ['13:00', '14:00', 0], ['12:00', '15:00', 1] ],
        'count' => 2,
        'tests' => {'08:00'=>0, '09:00'=>0,'09:30'=>0, '10:00'=>0,'10:30'=>0,'11:00'=>0,'12:00'=>1,'13:00'=>1, '14:00'=>1 }
    },
);

sub Main {
    ConnectTestSite();

    ok( Init(), "initialization" );

    my $Calendar = LoadObjectByPath($ROOTPATH);
    # start all test intervals
    my $LockingPeriodClass = LoadClassByAlias('LockingPeriod');
    foreach my $TimeTest (reverse @TESTS) {
        foreach my $Lock (@{$TimeTest->{'locks'}}) {
            $Calendar->setLock({
                'StartDate' => $Parser->parse_datetime('2007-10-15T'.$Lock->[0].':00'),
                'EndDate'   => $Parser->parse_datetime('2007-10-15T'.$Lock->[1].':00'),
                'IsLocked'  => $Lock->[2]
            });
        }
        my @Lockings = sort { $a->get('StartDate') <=> $a->get('EndDate') } @{$LockingPeriodClass->getObjects('ByCalendarID',[$Calendar->id])};
        ok( $TimeTest->{'count'} == scalar @Lockings, $TimeTest->{'name'} . ' count');
        if (exists $TimeTest->{'result'} && scalar @{$TimeTest->{'result'}} == scalar @Lockings) {
            my $ResultIsOk = 1;
            for(my $i=0; $i<scalar @Lockings; $i++) {
                my $S = $TimeTest->{'result'}[$i];
                my $I = $Lockings[$i];
                $ResultIsOk &= $Parser->parse_datetime('2007-10-15T'.$S->[0].':00') == $I->get('StartDate');
                $ResultIsOk &= $Parser->parse_datetime('2007-10-15T'.$S->[1].':00') == $I->get('EndDate');
                $ResultIsOk &= $S->[2] == $I->get('IsLocked');
            }
            ok($ResultIsOk, $TimeTest->{'name'}. ' result');
        }
        my $TimeIsOk = 1;
        foreach my $Time (sort keys %{$TimeTest->{'tests'}}) {
            $TimeIsOk &= $TimeTest->{'tests'}{$Time} == $Calendar->isLocked($Parser->parse_datetime('2007-10-15T'.$Time.':00'));
        }
        ok($TimeIsOk, $TimeTest->{'name'}. ' times');

        $_->delete foreach @Lockings;
        my $Lockings = $LockingPeriodClass->getObjects('ByCalendarID',[$Calendar->id]);
        ok( 0 == scalar @$Lockings, $TimeTest->{'name'} . ' delete all');
    }

    ok( Cleanup(), "cleanup" );

    DisconnectSite();
}

sub Cleanup {
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    Cleanup();
    LoadRootObject()->insertChild('Calendar', {'Alias'=>'Test'});
    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);
