use Test::More tests => 40;
use strict;
use DE_EPAGES::Core::API::Constants;
use DE_EPAGES::Core::API::Error qw( Error ExistsError GetError ResetError CleanupOnError VarsOnError );
local $SIG{__DIE__} = \&DE_EPAGES::Core::API::Error::DieHandler;
#
#
#
# DO NOT REMOVE ANY OF THOSE LINES; THESE ARE REQUIRED FOR COUNTING THE CORRECT
# ERROR LINE (NO FUNNY STUFF)
#
#
sub MyFunction {
    my ( $Name, $Value ) = @_;

    # line 17
    Error( 'MISSING_PARAMETER', { Parameter => 'Name' } ) unless defined $Name;

    # line 20
    Error( 'VALUE_TOO_LONG', { Length => length $Value, MaxLength => 25 } ) if length $Value > 25;

    # line 23
    die "Value too short" if length $Value < 3;

    return 'OK';
}

sub MyReEntryFunction {
    eval { MyFunction(); };
    Error(GetError());
}

sub MyReplaceDieHandlerFunction {
    local $SIG{__DIE__} = undef;
    eval { MyFunction('Native', '12'); };
    Error(GetError());
}

#ERRORDEFINITION
sub ErrorDefinition { return {
    'MISSING_PARAMETER' => 'Missing parameter #Parameter',
    'VALUE_TOO_LONG' => 'Value length #Length is larger than the allowed maximum #MaxLength'
}}
#ERRORDEFINITION END

my $DEBUG = 1;

# 1..2 - no error
my $Result = eval { MyFunction( 'Name', 'Value' ) };
ok( !ExistsError, "eval without error" );
ok( 'OK' eq $Result, "function result" );

# 3..8 - GetError
eval { $Result = MyFunction( undef, 'Value' ) };
ok( ExistsError(), "eval with error" );
my $Error = GetError();
is( ref $Error, 'DE_EPAGES::Core::API::Error', "Error is an Object reference" );
ok( $Error->isa('DE_EPAGES::Core::API::Error'), "class of error object" );
is( $Error->code, 'MISSING_PARAMETER', "error code" );
is( $Error->vars->{'Parameter'}, 'Name', "error variable" );
ok( 1, "dummy" );
ok( ExistsError(), "GetError does not reset" );

# 9..11 call stack
is( $Error->callers->[0]->[CALLER_LINE], 17, "line number" );
is( $Error->callers->[0]->[CALLER_SUBNAME], 'DE_EPAGES::Core::API::Error::Error', "Error() function name" );
is( $Error->callers->[1]->[CALLER_SUBNAME], 'main::MyFunction', "caller function name" );

# 12 ResetError
ResetError();
ok( !ExistsError(), "ResetError" );

# 13 shortMessage
ok( $Error->shortMessage eq 'Missing parameter Name', "shortMessage" );

# 14..16 longMessage
my $LongMessage = $Error->longMessage;
ok( $LongMessage =~ /^Missing parameter Name at \S*Error.t line 17.\n\n/, "longMessage (subject)" );
ok( $LongMessage =~ /sub\ main\:\:MyFunction\ called\ at\ \S*Error\.t\ line\ \d+\./x, "longMessage (caller)" );
ok( $LongMessage =~ /Vars{'Parameter'} = 'Name'/, "longMessage (vars)" );

# 17..20 die
eval { $Result = MyFunction( 'Name', 'V' ) };
ok( ExistsError(), "native 'die'" );
$Error = GetError();
is( $Error->code, 'NativeDie', "error code for native 'die'" );
my $msg = 'Value too short at '.__FILE__.' line 23.';
is( $Error->message, $msg, "message for native 'die'.");
is( $Error->shortMessage, 'Value too short', "short message for native 'die'.");

# 21..22 without $SIG{__DIE__}
my $Output;
local $SIG{__DIE__} = sub { $Output = shift; };
eval { $Result = MyFunction( undef, 'Value' ) };
ok( $Output =~ /^MISSING_PARAMETER: Missing parameter Name at \S*Error\.t line 17\./, "error without __DIE__ handler" );
eval { $Result = MyFunction( 'Name', 'V' ) };
ok( $Output =~ /^Value too short at \S*Error\.t line 23\./, "die without __DIE__ handler" );

# 23..24 re-entry stacks
local $SIG{__DIE__} = \&DE_EPAGES::Core::API::Error::DieHandler;
eval { $Result = MyFunction( undef, 'Value' ) };
$Error = ExistsError() ? GetError() : undef;
eval {
    MyReEntryFunction();
};
$Error = GetError();
my $ReEntries = $Error->reEntries;
ok (scalar @$ReEntries == 1, 're-entry found');
is ($ReEntries->[0][0][CALLER_LINE], 30, 're-entry at line 30');

# 25 local native die
eval { MyReplaceDieHandlerFunction(); };
$Error = GetError();
is ($Error->code, 'NativeDie', 'local native die');

# 26..27 syntax error
##no critic (ProhibitStringyEval)
eval "\n\n...;\nxxx;";
is( GetError()->shortMessage, 'syntax error', "syntax error: short message" );
ok( GetError()->message =~ /^syntax error at \(eval \d+\) line 3\.$/, "syntax error: message" );
#use critic
# 28..32 inner error
eval {
    eval {
        MyFunction( undef, 'Value' );
    };
    ok( ExistsError(), "caught inner error" );
    my $InnerError = GetError();
    Error( 'OuterError', { OuterVar => 'Value' }, $InnerError );
};
ok( ExistsError(), "caught outer error" );
my $OuterError = GetError();
is( GetError->code, 'OuterError', "outer error code" );
is( GetError->vars->{OuterVar}, 'Value', "outer error vars" );
ok( GetError->innerLongMessage =~ /^Missing parameter Name/, "innerLongMessage from error" );

# 33 inner error message
eval {
    Error( 'OuterError', { OuterVar => 'Value' }, 'TestMessage' );
};
ok( GetError->innerLongMessage eq 'TestMessage', "innerLongMessage from message" );

# VarsOnError
eval {
    eval {
        Error('Test', { Var_1 => 'Value_1' });
    };
    VarsOnError({ Var_2 => 'Value_2' });
};
ok( ExistsError(), "VarsOnError - ExistsError" );
is( GetError()->code, 'Test', "VarsOnError - code" );
is( GetError()->vars->{Var_2}, 'Value_2', "VarsOnError - vars" );

# CleanupOnError
eval {
    eval {
        Error('Test_Cleanup', { Var_1 => 'Value_1' });
    };
    CleanupOnError( sub {
        my ($Err) = @_;
        is( $Err->code, 'Test_Cleanup', "CleanupOnError - parameter" );
        $Err->addVars({ Var_3 => 'Value_3' });
    });
};
ok( ExistsError(), "CleanupOnError - ExistsError" );
is( GetError()->vars->{Var_3}, 'Value_3', "CleanupOnError - vars" );
