use Test::More tests => 117;
use strict;
use Config;
use Encode qw( decode );
use File::Temp qw( tempdir );
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::File qw(
    fsplit
    CreateDirectory
    DeleteFile
    DeleteDirectory
    RealPath
    ExistsFile
    ExistsFileCaseSensitive
    GetFileContent
    GetFileContentLines
    GetFileNames
    GetFileSize
    WriteFile
    LockFile
    IsFileNewer
    GetFileMD5Digest
    ConvertPathChars
    RelativePath
    SetAccessRights
    GetAccessRights

    CopyFile
    Text2Unix
    Text2Dos

    CompareMD5File
    DeleteDirectory
    GetDirectorySize
    DetectFileEncoding

    MoveFile
    ExistsDirectory

    RunOnDirectory
);
use DE_EPAGES::Core::API::Log qw(InitLog);
use DE_EPAGES::Core::API::Process; # avoid warning "Too late to run INIT block"

InitLog();

# RealPath
ok( RealPath('../Echo') eq '../Echo', 'RealPath ../Echo' );
ok( RealPath('Echo/../Die') eq 'Die', 'RealPath Echo/../Die' );
ok( RealPath('Echo/..') eq '',        'RealPath Echo/..' );

# example      ('/', 'roedel', '.txt') = fsplit('/roedel.txt');
my ($path, $name, $ext) = ();
($path, $name, $ext) = fsplit('/roedel.txt');
ok( $path eq '/',      'fsplit: path=' . $path );
ok( $name eq 'roedel', 'fsplit: name=' . $name );
ok( $ext  eq '.txt',   'fsplit: ext=' . $ext );

# example      ('/', 'roedel', undef )    = fsplit('/roedel');
($path, $name, $ext) = fsplit('/roedel');
ok( $path eq '/',       'fsplit: path is root' );
ok( $name eq 'roedel',  'fsplit: name=' . $name );
ok( defined $ext && $ext eq '', 'fsplit: no extension' );

# example      ('.', 'roedel', '.txt') = fsplit('roedel.txt');
($path, $name, $ext) = fsplit('roedel.txt');
ok( $path eq '',        'fsplit: path is empty' );
ok( $name eq 'roedel',  'fsplit: name=' . $name );
ok( $ext  eq '.txt',    'fsplit: ext=' . $ext );

# example      ('c:/epages', 'roedel', '.txt') = fsplit('c:/epages/roedel.txt');
($path, $name, $ext) = fsplit('c:/epages/roedel.txt');
ok( $path eq 'c:/epages', 'fsplit: DOS path='. $path );
ok( $name eq 'roedel',    'fsplit: DOS file name='. $name );
ok( $ext  eq '.txt',      'fsplit: DOS extension='. $ext );

# fsplit('c:/epages/roedel.en.txt');
($path, $name, $ext) = fsplit('c:/epages/roedel.en.txt');
ok( $path eq 'c:/epages', 'fsplit: path=' . $path );
ok( $name eq 'roedel.en', 'fsplit: name=' . $name );
ok( $ext  eq '.txt',      'fsplit: last extension=' . $ext );

# fsplit('\\\\test\\roedel.en.txt');
($path, $name, $ext) = fsplit('\\\\test\\roedel.en.txt');
ok( $path eq '\\\\test',  'fsplit: UNC path' );
ok( $name eq 'roedel.en', 'fsplit: UNC file name' );
ok( $ext  eq '.txt',      'fsplit: UNC extension'  );

# WriteFile
my $FileName  = 'TestFile.txt';
my $FileName2 = 'TestFile2.txt';
my $Content = "Line 1\nLine 2";
WriteFile( $FileName, \$Content );
ok( ExistsFile( $FileName ), "ExistsFile" );

# test newer file
sleep(1);
WriteFile( $FileName2, \$Content ); #write same file little bit later
ok (IsFileNewer($FileName, $FileName2), 'is newer');
ok (!IsFileNewer($FileName2, $FileName), 'is older');

eval { IsFileNewer('invalid.txt', $FileName2) };
ok( ExistsError(), "source doesn't exist" );
is( GetError()->code, 'FILE_NOT_FOUND', "source doesn't exist - error code" );

eval {
    ok( !IsFileNewer($FileName, 'invalid.txt'), "target doen't exist" );
};
ok( !ExistsError(), "target doesn't exist - no error" );

my $NativeSize = $Config{'osname'} eq 'MSWin32' ? 14 : 13;
ok( GetFileSize($FileName) == $NativeSize, "native operating system linefeeds" );
ok( ${GetFileContent($FileName)} eq $Content, "GetFileContent" );

my $aLines = GetFileContentLines($FileName);
ok( $aLines->[0] eq "Line 1\n", "GetFileContentLines 1" );
ok( $aLines->[1] eq 'Line 2', "GetFileContentLines 2" );

WriteFile( $FileName, \$Content, { binary => 1 } );
# Treatment of binary and text mode files is somewhat different between
# Unix and Windows
my $ExpectedBinarySize= $Config{'osname'} eq 'MSWin32' ? $NativeSize - 1 : $NativeSize;
ok( GetFileSize($FileName) == $ExpectedBinarySize, 'WriteFile binary mode' );

WriteFile( $FileName, \$Content, { binary => 0 } );
ok( GetFileSize($FileName) == $NativeSize, 'WriteFile text mode' );

# append
# without linefeeds to avoid win32/unix differences
my $ContentWithoutLinefeeds= 'MS Windows has carriage return + newline. Unix only has newline.';
WriteFile( $FileName, \$ContentWithoutLinefeeds );
WriteFile( $FileName, \$ContentWithoutLinefeeds, { 'append' => 1 } );
ok( GetFileSize($FileName) == 2 * length $ContentWithoutLinefeeds, 'append mode' );

# encoding
$Content = decode( 'iso-8859-1', "" );
WriteFile( $FileName, \$Content );
ok( GetFileSize($FileName) == 4, 'native operating system encoding (iso-1)' );

WriteFile( $FileName, \$Content, { 'utf8' => 1, 'flock' => 1 } );
ok( GetFileSize($FileName) == 8, "native Perl encoding (utf-8)" );
ok( ${GetFileContent($FileName)} ne $Content, "GetFileContent (normal)" );
ok( ${GetFileContent($FileName, { 'utf8' => 1, 'flock' => 1 })} eq $Content, "GetFileContent (utf-8)" );

# DeleteFile
DeleteFile( $FileName );
DeleteFile( $FileName2 );
ok( !ExistsFile( $FileName ), "DeleteFile" );

# DeleteDirectory [EPG-4777]
my $tmpdir= tempdir();
my $content= "hello world!";

WriteFile($tmpdir . '/Ostpark Getrnke Logo.jpg',           \$content, {binary => 0, syswrite => 1});
WriteFile($tmpdir . '/Ostpark Getrnke Logo-Briefkopf.jpg', \$content, {binary => 0, syswrite => 1});
WriteFile($tmpdir . '/Sem ttulo.jpg.XXXX',                 \$content, {binary => 0, syswrite => 1});
WriteFile($tmpdir . '/Logo diseojpg.XXXX',                 \$content, {binary => 0, syswrite => 1});

utf8::upgrade($tmpdir);
my $deletedDir= DeleteDirectory($tmpdir, {'recursive' => 1});
ok( $deletedDir == 1, 'DeleteDirectory' );

# bug RealPath
ok( RealPath('../../../Test/roedel.txt') eq '../../../Test/roedel.txt', 'not joined realpath' );
ok( RealPath('../../../Test/../Beta/tester.txt') eq '../../../Beta/tester.txt', 'joined realpath' );

# MD5 Hash for files
ok ( length(GetFileMD5Digest('File.t')) == 32, 'MD5Digest');

# bug 1345
my ($Path, $FName, $Extension) = fsplit('C:/Documents and Settings/sstock/Desktop/Aufr?en/topnavbarx');
ok ( defined $Extension && $Extension eq '', 'Extension empty' );

# ConvertPathChars
my $Alias = decode( 'iso-8859-1', '#IBN-08/25.1234.' );
ok( '_0023_IBN-08_002F_25_002E_1234_00FC__00F6__00DF__002E_' eq ConvertPathChars( $Alias ), "ConvertPathChars" );
my $ValidChars = '~$!+{}()[]';
ok( $ValidChars eq ConvertPathChars( $ValidChars ), "ConvertPathChars with valid characters" );

# copy text files and convert them to unix or win format
CopyFile('test.txt', 'testMD5Unix.txt');
Text2Unix('testMD5Unix.txt');

CopyFile('test.txt', 'testMD5Win.txt');
Text2Dos('testMD5Win.txt');

ok( 1== CompareMD5File('test.txt', 'b97503a70e7337b81543901749102b17'), "test test.txt with winMD5");
ok( 1== CompareMD5File('test.txt', '70260ca66a63f480f529f9ebeb18931e'), "test test.txt with unixMD5");

ok( 1== CompareMD5File('testMD5Win.txt', 'b97503a70e7337b81543901749102b17'), "test testMD5Win.txt with winMD5");
ok( 1== CompareMD5File('testMD5Win.txt', '70260ca66a63f480f529f9ebeb18931e'), "test testMD5Win.txt with unixMD5");

ok( 1== CompareMD5File('testMD5Unix.txt', 'b97503a70e7337b81543901749102b17'), "test testMD5Unix.txt with winMD5");
ok( 1== CompareMD5File('testMD5Unix.txt', '70260ca66a63f480f529f9ebeb18931e'), "test testMD5Unix.txt with unixMD5");

# CreateDirectory / DeleteDirectory
DeleteDirectory( 't1', { recursive => 1 } );
ok( !ExistsDirectory('t1'), "before CreateDirectory" );

CreateDirectory( 't1' );
ok( ExistsDirectory('t1'), "CreateDirectory simple" );

DeleteDirectory( 't1' );
ok( !ExistsDirectory('t1'), "DeleteDirectory simple" );

eval {
    CreateDirectory( 't1/t2', { recursive => 0 } );
};
ok( ExistsError(), "CreateDirectory not recursive" );
is( GetError()->code, 'DIR_CREATE_FAILED', "CreateDirectory not recursive" );
ok( !ExistsDirectory('t1/t2'), "CreateDirectory not recursive" );
DeleteDirectory( 't1', { recursive => 1 } );

CreateDirectory( 't1/t2' );
ok( ExistsDirectory('t1/t2'), "CreateDirectory recursive" );
DeleteDirectory( 't1', { recursive => 1 } );

SKIP:
{
    skip ( "CreateDirectory doesn't fail on Linux :)", 4 ) if $Config{'osname'} ne 'MSWin32';
    eval {
        CreateDirectory( '*' );
    };
    ok( ExistsError(), "CreateDirectory with invalid file name" );
    is( GetError()->code, 'DIR_CREATE_FAILED', "CreateDirectory with invalid file name" );
    is( GetError()->vars->{'Directory'}, '*', "Directory name in error object" );
    ok( !ExistsDirectory('t1/t2'), "CreateDirectory with invalid file name" );
}

CreateDirectory( 'ImageTestDirectorySize' );
ok( -d 'ImageTestDirectorySize', "CreateDirectory" );
my @Files = qw(
    compare_NewImage1.jpg
    compare_NewImage2.gif
    compare_1_ScaleImage1.jpg
    compare_1_ScaleImage3.jpg
);
CopyFile( "Image/$_", "ImageTestDirectorySize/$_" ) foreach @Files;

my $DirSize = -s 'ImageTestDirectorySize';
ok( $DirSize + 57779 == GetDirectorySize('ImageTestDirectorySize'), 'directory size');
ok( $DirSize + 69632 == GetDirectorySize('ImageTestDirectorySize', 4*1024), 'directory size with 4k blocks ');
DeleteDirectory( 'ImageTestDirectorySize', {'recursive' => 1} );
ok( ! -d 'ImageTestDirectorySize', "DeleteDirectory('ImageTestDirectorySize')" );

# LockFile
my $FileHandle;
open( $FileHandle, '>>', $FileName ) or die "Can't open file: $!";
LockFile( $FileHandle, 'w', sub {
    ok( 1, 'LockFile' );
    seek( $FileHandle, 0, 2 ) or die $!;
    print $FileHandle "...\n";
});
close $FileHandle or die $!;
DeleteFile( $FileName );

# DetectFileEncoding
ok( !defined DetectFileEncoding( 'BOM/nobom.txt' ), "DetectFileEncoding: no BOM" );
ok( 'utf-7' eq DetectFileEncoding( 'BOM/utf-7.txt' ), "DetectFileEncoding: utf-7" );
ok( 'utf-8' eq DetectFileEncoding( 'BOM/utf-8.txt' ), "DetectFileEncoding: utf-8" );
ok( 'utf16le' eq DetectFileEncoding( 'BOM/utf16le.txt' ), "DetectFileEncoding: utf16le" );
ok( 'utf16be' eq DetectFileEncoding( 'BOM/utf16be.txt' ), "DetectFileEncoding: utf16be" );

# GetFileContent, usebom
$Content = ${ GetFileContent( 'BOM/nobom.txt', { 'utf8' => 1 } ) };
ok( 22 == length $Content, "GetFileContent utf8" );
ok( 32 == length ${GetFileContent( 'BOM/nobom.txt', { 'usebom' => 1 } )}, "GetFileContent usebom nobom" );
ok( $Content eq ${GetFileContent( 'BOM/utf-7.txt', { 'usebom' => 1 } )}, "GetFileContent usebom utf-7" );
ok( $Content eq ${GetFileContent( 'BOM/utf-8.txt', { 'usebom' => 1 } )}, "GetFileContent usebom utf-8" );
ok( $Content eq ${GetFileContent( 'BOM/utf16le.txt', { 'usebom' => 1 } )}, "GetFileContent usebom utf16le" );
ok( $Content eq ${GetFileContent( 'BOM/utf16be.txt', { 'usebom' => 1 } )}, "GetFileContent usebom utf16be" );

# GetFileContentLines, usebom
ok( $Content eq join('', @{GetFileContentLines( 'BOM/nobom.txt', { 'utf8' => 1 } )}), "GetFileContentLines usebom nobom" );
ok( $Content eq join('', @{GetFileContentLines( 'BOM/utf-7.txt', { 'usebom' => 1 } )}), "GetFileContentLines usebom utf-7" );
ok( $Content eq join('', @{GetFileContentLines( 'BOM/utf-8.txt', { 'usebom' => 1 } )}), "GetFileContentLines usebom utf-8" );
ok( $Content eq join('', @{GetFileContentLines( 'BOM/utf16le.txt', { 'usebom' => 1 } )}), "GetFileContentLines usebom utf16le" );
ok( $Content eq join('', @{GetFileContentLines( 'BOM/utf16be.txt', { 'usebom' => 1 } )}), "GetFileContentLines usebom utf16be" );

# WriteFile, usebom
WriteFile( $FileName, \$Content, { 'utf8' => 1 } );
ok( GetFileSize($FileName) == GetFileSize('BOM/nobom.txt'), "WriteFile utf8" );
WriteFile( $FileName, \$Content, { 'utf8' => 1, 'usebom' => 1 } );
ok( GetFileSize($FileName) == GetFileSize('BOM/utf-8.txt'), "WriteFile utf8 usebom" );
WriteFile( $FileName, \$Content, { 'encoding' => 'utf-7', 'usebom' => 1 } );
ok( GetFileSize($FileName) == GetFileSize('BOM/utf-7.txt'), "WriteFile utf-7 usebom" );
WriteFile( $FileName, \$Content, { 'encoding' => 'utf-8', 'usebom' => 1 } );
ok( GetFileSize($FileName) == GetFileSize('BOM/utf-8.txt'), "WriteFile utf-8 usebom" );
WriteFile( $FileName, \$Content, { 'encoding' => 'utf16le', 'usebom' => 1 } );
ok( GetFileSize($FileName) == GetFileSize('BOM/utf16le.txt'), "WriteFile utf16le usebom" );
WriteFile( $FileName, \$Content, { 'encoding' => 'utf16be', 'usebom' => 1 } );
ok( GetFileSize($FileName) == GetFileSize('BOM/utf16be.txt'), "WriteFile utf16be usebom" );
WriteFile( $FileName, \$Content, { 'encoding' => 'utf-8', 'usebom' => 1, 'syswrite' => 1 } );
ok( GetFileSize($FileName) == 35, "WriteFile utf-8 usebom syswrite" );
DeleteFile( $FileName );

# RelativePath
ok( "../SubFolder2" eq RelativePath('C:\Folder\SubFolder1', 'C:\Folder\SubFolder2'), "RelativePath - sibling" );
ok( "" eq RelativePath('C:\Folder\SubFolder1', 'C:\Folder\SubFolder1'), "RelativePath - identic" );
ok( "../../../SubFolder2/SubSubFolder" eq RelativePath('C:\Folder\SubFolder1\SubSubFolder\SubSubSubFolder', 'C:\Folder\SubFolder2\SubSubFolder'), "RelativePath - deep" );
ok( "SubSubFolder/SubSubSubFolder" eq RelativePath('C:\Folder\SubFolder', 'C:\Folder\SubFolder\SubSubFolder\SubSubSubFolder'), "RelativePath - base is deep parent" );
ok( "../../" eq RelativePath('C:\Folder\SubFolder\SubSubFolder\SubSubSubFolder', 'C:\Folder\SubFolder'), "RelativePath - base is deep child" );
ok( "D:/Folder/SubFolder" eq RelativePath('C:\Folder', 'D:\Folder\SubFolder'), "RelativePath - different drive letter (Windows)" );
ok( "/hdb/tmp" eq RelativePath('/hda/epages', '/hdb/tmp'), "RelativePath - different drive letter (*nix)" );

CopyFile('test.txt', 'tEsT84.txt');
ok( ExistsFileCaseSensitive($ENV{'EPAGES_CARTRIDGES'} . '/DE_EPAGES/Core/t/tEsT84.txt'), "Found file 'tEsT84.txt' case sensitive" );
ok(! ExistsFileCaseSensitive($ENV{'EPAGES_CARTRIDGES'} . '/DE_EPAGES/Core/t/test84.txt'), "file 'test84.txt' correctly ignored");

# MoveFile
CopyFile('test.txt', 'movetest.txt');
MoveFile('movetest.txt', 'tEsT42t.txt');
ok(!ExistsFile('movetest.txt'), "movefile: old file is gone");
ok( ExistsFile('tEsT42t.txt'), "movefile: new file is there");

# test for EPG-13034
MoveFile('tEsT42t.txt', $ENV{'EPAGES_CARTRIDGES'} . '/DE_EPAGES/Core/t/srv/epages/eproot/Shared/WebRoot/Store/Shops/coshop/MediaGallery/Screen_Werterabatt.jpg', { create_directory => 1 });
ok(!ExistsFile('tEsT42t.txt'), "movefile: old file is gone");
ok( ExistsFile($ENV{'EPAGES_CARTRIDGES'} . '/DE_EPAGES/Core/t/srv/epages/eproot/Shared/WebRoot/Store/Shops/coshop/MediaGallery/Screen_Werterabatt.jpg'), "movefile: new file is there");

MoveFile($ENV{'EPAGES_CARTRIDGES'} . '/DE_EPAGES/Core/t/srv/epages/eproot/Shared/WebRoot/Store/Shops/coshop/MediaGallery/Screen_Werterabatt.jpg', $ENV{'EPAGES_CARTRIDGES'} . '/DE_EPAGES/Core/t/srv/epages/eproot/Shared/WebRoot/Store/Shops/coshop/MediaGallery/_Screen_Werterabatt.jpg');
ok(!ExistsFile($ENV{'EPAGES_CARTRIDGES'} . '/DE_EPAGES/Core/t/srv/epages/eproot/Shared/WebRoot/Store/Shops/coshop/MediaGallery/Screen_Werterabatt.jpg'), "movefile: old file is gone");
ok( ExistsFile($ENV{'EPAGES_CARTRIDGES'} . '/DE_EPAGES/Core/t/srv/epages/eproot/Shared/WebRoot/Store/Shops/coshop/MediaGallery/_Screen_Werterabatt.jpg'), "movefile: new file is there");

DeleteDirectory($ENV{'EPAGES_CARTRIDGES'} . '/DE_EPAGES/Core/t/srv', { recursive => 1 });
ok(!ExistsDirectory($ENV{'EPAGES_CARTRIDGES'} . '/DE_EPAGES/Core/t/srv'), "DeleteDirectory: test dir is gone");

@Files = grep { /\.txt$/ } RunOnDirectory( 'BOM', sub { @{ GetFileNames('.') } } );
is( scalar @Files, 5, "RunOnDirectory BOM" );

eval {
    @Files = grep { /\.txt$/ } RunOnDirectory( 'wrongDir', sub { @{ GetFileNames('.') } } );
};
ok( ExistsError() && GetError()->code eq 'CHANGE_DIR_FAILED', "RunOnDirectory: directory not found" );

SKIP:
{
     skip ( ': because file access right mechanism only works on Linux.', 7 ) if $Config{'osname'} eq 'MSWin32';

     # WriteFile
     $FileName= 'AccessRightFile.txt';
     $Content= 'Line 1';
     WriteFile( $FileName, \$Content );

     # SetAccessRights
     SetAccessRights( 'AccessRightFile.txt', { 'Mode' => oct(755), 'UserID' => 1111, 'GroupID' => 2222 } );

     # Check mode, user and group
     my $hAccess= GetAccessRights( 'AccessRightFile.txt' );
     ok( $hAccess->{'Mode'} == oct(755), 'File mode equals expected value.' );
     ok( $hAccess->{'UserID'} == 1111, 'File is owned by expected user id.' );
     ok( $hAccess->{'GroupID'} == 2222, 'File belongs to expected group id.' );

     # EPG-14671: SetAccessRights ($;$) with explicit setting of User(ID), GroupID in $hFlags results in Mode == 0
     SetAccessRights( 'AccessRightFile.txt', { 'UserID' => 2222, 'GroupID' => 3333 } );
     $hAccess= GetAccessRights( 'AccessRightFile.txt' );
     ok( $hAccess->{'Mode'} == oct(644), 'File mode equals expected value.' );
     ok( $hAccess->{'UserID'} == 2222, 'File is owned by expected user id.' );
     ok( $hAccess->{'GroupID'} == 3333, 'File belongs to expected group id.' );

     # DeleteFile
     DeleteFile( $FileName );
     ok( !ExistsFile( $FileName ), 'File was deleted.' );
}
