##no critic qw(ProhibitExcessMainComplexity)
use Test::Simple tests => 52;
use strict;
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::TestParameter qw( TestObject TestString TestIpAddress
                                            TestDomain TestUrl);

# TestString

eval { TestString('Name', 'String'); };
ok( !ExistsError(), "TestString('Name', 'String ok')" );

eval { TestString(); };
ok( ExistsError() && GetError->code eq 'PARAMETER_MISSING', "TestString()" );

eval { TestString('Name'); };
ok( ExistsError() && GetError->code eq 'PARAMETER_MISSING', "TestString('Name')" );

eval { TestString('Name', undef); };
ok( ExistsError() && GetError->code eq 'PARAMETER_MISSING', "TestString('Name', undef)" );

eval { TestString('Name', ''); };
ok( ExistsError() && GetError->code eq 'STRING_EMPTY', "TestString('Name', '')" );

eval { TestString('Name', '   '); };
ok( ExistsError() && GetError->code eq 'STRING_EMPTY', "TestString('Name', '   ')" );

eval { TestString('Name', 'too short', 10); };
ok( ExistsError() && GetError->code eq 'STRING_TOO_SHORT', "TestString('Name', 'too short')" );

eval { TestString('Name', 'too long', undef, 3); };
ok( ExistsError() && GetError->code eq 'STRING_TOO_LONG', "TestString('Name', 'too long')" );

use utf8;
my $Uft8String = 'Радуга';
no utf8;

eval { TestString('Name', $Uft8String, undef, 5); };
ok( ExistsError() && GetError->code eq 'STRING_TOO_LONG', "utf8 string is too long" );

eval { TestString('Name', $Uft8String, 7, undef); };
ok( ExistsError() && GetError->code eq 'STRING_TOO_SHORT', "utf8 string is too short" );

eval { TestString('Name', $Uft8String, 6, 6); };
ok( !ExistsError(), "utf8 string is ok" );


# TestObject

eval { TestObject(); };
ok( ExistsError() && GetError->code eq 'PARAMETER_MISSING', "TestObject()" );

eval { TestObject('Name'); };
ok( ExistsError() && GetError->code eq 'PARAMETER_MISSING', "TestObject('Name')" );

eval { TestObject('Name', undef); };
ok( ExistsError() && GetError->code eq 'PARAMETER_MISSING', "TestObject('Name', undef)" );

eval { TestObject('Name', 'String'); };
ok( ExistsError() && GetError->code eq 'PARAMETER_NO_OBJECT', "TestObject('Name', 'String')" );

eval { TestObject('Name', []); };
ok( ExistsError() && GetError->code eq 'PARAMETER_NO_OBJECT', "TestObject('Name', [])" );

my $Object = TestPackage->new;
eval { TestObject('Name', $Object); };
ok( !ExistsError(), "TestObject('Name', \$Object)" );

eval { TestObject('Name', $Object, 'TestPackage'); };
ok( !ExistsError(), "TestObject('Name', \$Object, 'TestPackage')" );

eval { TestObject('Name', $Object, 'BasePackage'); };
ok( !ExistsError(), "TestObject('Name', \$Object, 'BasePackage')" );

eval { TestObject('Name', $Object, 'WrongPackage'); };
ok( ExistsError() && GetError->code eq 'WRONG_CLASS', "TestObject('Name', \$Object, 'WrongPackage')" );

# TestIpAddress

my %IpTests = (
    'foo'                 => 'VOID_IP_ADDRESS',
    'foo.bar'             => 'VOID_IP_ADDRESS',
    '1.2.3.4.5'           => 'VOID_IPv4_ADDRESS',
    '123.234.156.103'     => '',
    '123.234.256.103'     => 'VOID_IPv4_ADDRESS',
    '::1'                 => '',
    '1:2:3'               => 'VOID_IPv6_ADDRESS',
    '::ffff:1.2.3.4'      => '',
    '1:2:3:4:5:6:1.2.3.4' => 'VOID_IPv6_ADDRESS',
    '::ffff:1.2.3'        => 'VOID_IPv6_ADDRESS',
    '1:2:3:4:5:6:7:8'     => '',
    '1:2:3:4:5:6:7:x'     => 'VOID_IP_ADDRESS',
    );

foreach my $Key (keys %IpTests) {
    my $ReturnString = $IpTests{$Key};
    $ReturnString = 'VALID' unless $ReturnString;
    eval { TestIpAddress($ReturnString, $Key); };
    my $Msg = ExistsError() ? GetError->message : 'VALID';
    $Msg =~ s/.*\((.*?)[:\)].*/$1/sx;
    my $OK = $IpTests{$Key} ? $Msg eq $IpTests{$Key} : !ExistsError();
    ok($OK, "TestIpAddress('$ReturnString', '$Key') -> $Msg" );
}

# TestDomain

my %DomainTests = (
    'foo.bar'       => '',
    '0.1.2.3'       => '',
    '123reg.co.uk'  => '',
    '-epages.de'    => 'DOMAIN_BOUNDARY_HYPHEN',
    'www.l-.pt'     => 'DOMAIN_BOUNDARY_HYPHEN',
    'www..pt'       => 'DOMAIN_ATOM_EMPTY',
    'epages_.de'    => 'DOMAIN_CHARS_VOID',
    'm\x{FC}ller.de'=> 'DOMAIN_CHARS_VOID',
    'EightMegabytesAndConstantlySwappingEmacsManualsAreCrypticandSurreal' => 'DOMAIN_ATOM_GT_63_CHARS',
    'Each.Manuals.Audience.is.Completely.Stupified.Emacs.Means.A.Crappy.Screen.Eventually.Munches.All.Computer.Storage.Even.My.Aunt.Crashes.the.System.Eradication.of.Memory.Accomplished.with.Complete.Simplicity.Elsewhere.Maybe.Alternative.Civilizations.Survive' => 'DOMAIN_NAME_GT_253_CHARS',
    );

foreach my $Key (keys %DomainTests) {
    my $ReturnString = $DomainTests{$Key};
    $ReturnString = 'VALID' unless $ReturnString;
    eval { TestDomain($ReturnString, $Key); };
    my $Msg = ExistsError() ? GetError->message : 'VALID';
    $Msg =~ s/.*\((.*?)[:\)].*/$1/sx;
    my $OK = $DomainTests{$Key} ? $Msg eq $DomainTests{$Key} : !ExistsError();
    ok($OK, "TestDomainAddress('$ReturnString', '$Key') -> $Msg" );
}

# TestUrl

my %UrlTests = (
    'xyz://.~%ff!$&()*+,;:=-@[::1]:888?xyz' => '',
    'xyz://user:password@[::1]:8888/xyz'    => '',
    'xyz://user:password@[::1]:8888#xyz'    => '',
    'xyz://example.com&story=news@10.0.0.1' => '',
    'xyz:/user:password@[::1]:8888/xyz'     => 'URL_FORMAT_VOID',
    'xyz://user:pass@word@[::1]:8888/xyz'   => 'URL_USERINFO_VOID_CHARS',
    'xyz://user:password@m\x{FC}ller.de/xyz/' => 'DOMAIN_CHARS_VOID',
    'xyz://user:password@-epages.de/xyz/'   => 'DOMAIN_BOUNDARY_HYPHEN',
    'xyz://user%xx@epages.de/xyz/'          => 'URL_USERINFO_VOID_%HEX',
    'xyz://m\x{FC}ller@epages.de/xyz/'      => 'URL_USERINFO_VOID_CHARS',
    );

foreach my $Key (keys %UrlTests) {
    my $ReturnString = $UrlTests{$Key};
    $ReturnString = 'VALID' unless $ReturnString;
    eval { TestUrl($ReturnString, $Key); };
    my $Msg = ExistsError() ? GetError->message : 'VALID';
    $Msg =~ s/.*\((.*?)[:\)].*/$1/sx;
    my $OK = $UrlTests{$Key} ? $Msg eq $UrlTests{$Key} : !ExistsError();
    ok($OK, "TestUrl('$ReturnString', '$Key') -> $Msg" );
}

#==============================================================================
package BasePackage;

sub new {
    my $class = shift;
    return bless { @_ }, $class;
}

#==============================================================================
##no critic (ProhibitMultiplePackages)
package TestPackage;
BEGIN {
    @TestPackage::ISA = qw( BasePackage );
}

sub new {
    my $class = shift;
    return bless { @_ }, $class;
}
##use critic
