#!perl -w
use Test::More tests => 34;
use strict;
use DE_EPAGES::Core::API::Constants;
use DE_EPAGES::Core::API::Error qw( Error GetError );
use DE_EPAGES::Core::API::Log qw( InitLog );
use DE_EPAGES::Core::API::PerlTools qw(IsA);
use DE_EPAGES::Core::API::Warning qw( Warning BlockWarnings );
use Log::Log4perl::Layout;
use Log::Log4perl::Level;

local $SIG{__DIE__}  = \&DE_EPAGES::Core::API::Error::DieHandler;
local $SIG{__WARN__} = \&DE_EPAGES::Core::API::Warning::WarnHandler;
# DO NOT REMOVE ANY OF THOSE LINES; THESE ARE REQUIRED FOR COUNTING THE CORRECT 
# ERROR LINE (NO FUNNY STUFF)
#
sub MyFunction {
    my ( $Name, $Value ) = @_;

    # line 21
    Warning( 'MISSING_PARAMETER', { Parameter => 'Name' } ) unless defined $Name;

    # line 24
    Warning( 'VALUE_TOO_LONG', { Length => length $Value, MaxLength => 25 } ) if length $Value > 25;

    # line 27
    warn "Value too short" if length $Value < 3;
    die "Value is 26" if $Value eq 'sechs';
    return 'OK';
}

sub MyReEntryFunction {
    eval { MyFunction( 'Name', 'sechs' ); };

    # line 36
    Warning(GetError());
}

#ERRORDEFINITION
sub ErrorDefinition { return {
    'MISSING_PARAMETER' => 'Missing parameter #Parameter',
    'VALUE_TOO_LONG' => 'Value length #Length is larger than the allowed maximum #MaxLength'
}}
#ERRORDEFINITION END

InitLog();

# 1..2 - no warning
my $Result = undef;
my $aWarnings = BlockWarnings( sub { $Result = MyFunction( 'Name', 'Value' ); });
ok( (not scalar @$aWarnings), "without warning" );
ok( 'OK' eq $Result, "function result" );

# 3..6 - GetWarning
$aWarnings = BlockWarnings( sub { $Result = MyFunction( undef, 'Value' ) });
ok( scalar @$aWarnings, "with warning" );
my $Warning = $aWarnings->[0];
ok( IsA( $aWarnings->[0], 'DE_EPAGES::Core::API::Warning' ), "class of warning object" );
ok( $Warning->code eq 'MISSING_PARAMETER', "warning code" );
ok( $Warning->vars->{'Parameter'} eq 'Name', "warning variable" );

# 7..9 call stack
is( $Warning->callers->[0]->[CALLER_LINE], 21, "line number" );
is( $Warning->callers->[0]->[CALLER_SUBNAME], 'DE_EPAGES::Core::API::Warning::Warning', "Warning() function name" );
is( $Warning->callers->[1]->[CALLER_SUBNAME], 'main::MyFunction', "caller function name" );

# 10 shortMessage
ok( $Warning->shortMessage eq 'Missing parameter Name', "shortMessage" );

# 11..13 longMessage
my $LongMessage = $Warning->longMessage;
like( $LongMessage, qr/^Missing parameter Name at \S*Warning.t line 21.\n\n/, "longMessage (subject)" );  ##no critic(RequireExtendedFormatting)
like( $LongMessage, qr/sub main\:\:MyFunction called at \S*Warning\.t line \d+\./, "longMessage (caller)" );
like( $LongMessage, qr/Vars{'Parameter'} = 'Name'/, "longMessage (vars)" );

my $log = Log::Log4perl->get_logger("main::MyFunction");
my $appender = Log::Log4perl::Appender->new( "MY::StringAppender", name => "stringlog" );
$appender->layout( Log::Log4perl::Layout::PatternLayout->new("%m") );
$log->add_appender($appender);
$log->level($INFO);
Log::Log4perl->get_logger("DE_EPAGES::Core::API::Log::LogWarnings")->level($WARN);

# BlockWarnings without filter
$aWarnings = BlockWarnings( sub {
    $Result = MyFunction( 'Name', 'V' );
    $Result = MyFunction( 'Name', 'V' );
});
is( $appender->string, '', "BlockWarnings: nothing is logged" );
is( scalar @$aWarnings, 2, "BlockWarnings: warnings collected" );
$Warning = $aWarnings->[0];
is( $Warning->code, 'NativeWarn', "BlockWarnings: warning code for native 'warn'" );
is( $Warning->message, 'Value too short at '.__FILE__.' line 27.', "BlockWarnings: warning message for native 'warn'" );
is( $Warning->shortMessage, 'Value too short', "BlockWarnings: short message for native 'warn'"  );
is( scalar @{$Warning->callers}, 5, "BlockWarnings: stack depth" );

is( $Warning->callers->[0]->[CALLER_LINE], 27, "BlockWarnings: line number" );
is( $Warning->callers->[0]->[CALLER_SUBNAME], 'DE_EPAGES::Core::API::Warning::WarnHandler', "WarnHandler() function name" );
like( $Warning->callers->[0]->[CALLER_FILENAME], qr/\bWarning\.t$/, "BlockWarnings: script file name" );

# BlockWarnings with filter
$appender->reset;
$aWarnings = BlockWarnings( sub {
    $Result = MyFunction( undef, 'V' );
}, [qw( MISSING_PARAMETER )]);
is( $appender->string, 'Value too short at '.__FILE__." line 27.\n", "BlockWarnings with filter: log message of native warning" );
is( scalar @$aWarnings, 1, "BlockWarnings with filter: warnings collected" );
$Warning = $aWarnings->[0];
is( $Warning->code, 'MISSING_PARAMETER', "BlockWarnings with filter: warning code" );
is( $Warning->shortMessage, "Missing parameter Name", "BlockWarnings with filter: short message'"  );
is( scalar @{$Warning->callers}, 5, "BlockWarnings with filter: stack depth" );

# log warning with stack trace
$appender->reset;
Log::Log4perl->get_logger("DE_EPAGES::Core::API::Log::LogWarnings")->level($DEBUG);
MyFunction( undef, 'V' );
like( $appender->string, qr/Code = MISSING_PARAMETER/, "log message contains warning code" );
like( $appender->string, qr/sub main::MyFunction called at/, "log message contains stack trace" );

# without $SIG{__WARN__}
my $Output;
local $SIG{__WARN__} = sub { $Output = shift; };
MyFunction( undef, 'Value' );
ok( $Output =~ /^ISWARNING\: Missing parameter Name at \S*Warning\.t line 21\./, "warning without __WARN__ handler" );
MyFunction( 'Name', 'V' );
ok( $Output =~ /^Value too short at \S*Warning\.t line 27/, "warn without __WARN__ handler" );

# Error To Warnings
$aWarnings = BlockWarnings( sub { MyReEntryFunction() });
ok( IsA( $aWarnings->[0], 'DE_EPAGES::Core::API::Warning' ), "error is now a warning" );
is( scalar @{$aWarnings->[0]->reEntries}, 1, "re-entry logged" );
is( $aWarnings->[0]->reEntries->[0][0]->[CALLER_LINE], 36, "re-entry at line 36" );


#========================================================================================
# package      MY::StringAppender
# description  catches any string written to the log file
#========================================================================================
package MY::StringAppender;

use strict;

sub new {
    my($class, @options) = @_;

    return bless { @options, 'string' => '' }, $class;
}

sub log {
    my $self = shift;
    my %params = @_;

#print "LOG $params{message}" . "\n";
    $self->{'string'} .= $params{message} . "\n";
}

sub string {
    my $self = shift;
    return $self->{'string'};
}

sub reset {
    my $self = shift;
    $self->{'string'} = '';
}

1;
