/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

	epages.lang.array $Revision: 1.29.52.2 $

*/
dojo.provide('epages.lang.array');
dojo.require('epages.string');

$A = function (/*Object?*/arrayLike, /*Boolean?*/asCopy) {
	// summary:
	//    returns a new array, same like new epages.lang.Array
	// example:
	//  | hash = $A(['value1', 'value2']);
	//  | // returns epages.lang.Array
	return new epages.lang.Array(arrayLike, asCopy);
};

epages.lang.Array = /*epages.lang.Array*/ function (/*Object?*/arrayLike, /*Boolean?*/asCopy) {
	// summary:
	//    returns a new array
	// example:
	//  | array = epages.lang.Array(['value1', 'value2']);
	//  | // returns epages.lang.Array
	if (asCopy === undefined) asCopy = false;
	var results = [];
	if (arrayLike !== undefined) {
		if (arrayLike._data !== undefined) {
			arrayLike = arrayLike._data;
		}
		if (asCopy) {
			for (var i = 0, length = arrayLike.length; i < length; i++)
				results.push(arrayLike[i]);
		} else {
			results = arrayLike;
		}
	}
	this._data  = results;
	this.length = results.length;
};

epages.lang.Array.prototype = {
	_data: undefined,
	length: 0,

	get: function (/*int*/index) {
		return this._data[index];
	},

	set: function (/*int*/index, /*mixed*/value) {
		this._data[index] = value;
	},

	insertAt: function (/*int*/index, /*mixed*/value) {
		if (!index) {
			this.unshift(value);
		} else if (index == this.length) {
			this.push(value);
		} else {
			var c = this._data;
			for(var i=this.length; i > index; i--) c[i] = c[i-1];
			c[index] = value;
			this.length = c.length;
		}
	},

	push: function (/*arguments*/) {
		// summary:
		//    adds elements to this array
		// example:
		//  | array = new epages.lang.Array(['value1', 'value2']);
		//  | array.push('value3', 'value4');
		//  | array.push('value5');
		for (var i = 0, length = arguments.length; i < length; i++) {
			this._data.push(arguments[i]);
		}
		this.length = this._data.length;
	},

	pop: function (/*arguments*/) {
		// summary:
		//    pops the last element from this array
		// returns: 
		//    removed element
		// example:
		//  | array = new epages.lang.Array(['value1', 'value2']);
		//  | array.pop();
		//  | //returns 'value2'
		var el = this._data.pop();
		this.length = this._data.length;
		return el;
	},

	unshift: function (/*arguments*/) {
		// summary:
		//    adds elements to this array at the first position
		// example:
		//  | array = new epages.lang.Array(['value1', 'value2']);
		//  | array.unshift('value3', 'value4');
		//  | // returns ['value4', 'value3','value1', 'value2']
		for (var i = 0, length = arguments.length; i < length; i++) {
			this._data.unshift(arguments[i]);
		}
		this.length=this._data.length;
	},

	shift: function () {
		// summary:
		//    removes an elements from the beginning of this array
		// returns: 
		//    removed element
		var el = this._data.shift();
		this.length = this._data.length;
		return el;
	},

	grep: function (/*Function*/callback, /*Object?*/thisObject) {
		// summary:
		//    returns a new Array with those items from arr that match the
		//    condition implemented by callback.thisObject may be used to
		//    scope the call to callback.
		// example:
		//  | array = new epages.lang.Array([1, 2, 3, 4]);
		//  | array2 = array.grep( function(item){ return item>1; });
		//  | // returns [2, 3, 4]
		if (thisObject === undefined)
			return dojo.filter(this._data, callback); // Array
		return dojo.filter(this._data, callback, thisObject); // Array
	},

	map: function (/*Function*/callback, /*Object?*/thisObject) {
		// summary:
		//    returns calls for each item of this Array the given callback function and collect the return values.
		// example:
		//  | array = new epages.lang.Array([1, 2, 3, 4]);
		//  | var newArray = array.map( function(item){ return item + 1; });
		//  | // newArray = [2,3,4,5]
		return dojo.map(this._data, callback, thisObject); //Array
	},

	each: function (/*Function*/callback, /*Object?*/thisObject) {
		// summary:
		//    returns calls for each item of this Array the given callback function.
		// example:
		//  | array = new epages.lang.Array([1, 2, 3, 4]);
		//  | var sum = 0;
		//  | array.each( function(item){ sum += item; });
		//  | // sum = 10
		dojo.map(this._data, callback, thisObject);
		return; //void
	},

	match: function (/*RegExpr*/ pattern) {
		// summary:
		//    returns a new Array with those items from arr that match the
		//    condition (matches with the given pattern).
		// example:
		//  | array = new epages.lang.Array(['Test', 'Array', 'Hash', 'epages']);
		//  | array2 = array.match(/e/);
		//  | // returns ['Test', 'epages']
		var result = [];
		for (var i = 0, length = this._data.length; i < length; i++) {
			var element = this._data[i];
			if (typeof element.match == 'function' && element.match(pattern)) result.push(element);
		}
		return result; //Array
	},

	exists: function (/*Object*/element) {
		// summary:
		//    returns true if an item exists in the array,
		//    otherwise false.
		// example:
		//  | array = new epages.lang.Array(['Test', 'Array', 'Hash', 'epages']);
		//  | isElement = array.exists('e');
		//  | // returns false
		return (this.find(element) !== null); //boolean
	},

	find: function (/*Object*/element) {
		// summary:
		//    returns the index of the element, if it exists in the array
		// example:
		//  | array = new epages.lang.Array(['Test', 'Array', 'Hash', 'epages']);
		//  | isElement = array.find('Array');
		//  | // returns 1
		for (var i = 0, length = this._data.length; i < length; i++) {
			if (element === this._data[i]) return i; // integer or null
		}
		return null; // integer or null
	},

	remove: function (/*integer*/index) {
		// summary:
		//    removes the element on position >index
		// example:
		//  | array = new epages.lang.Array(['Test', 'Array', 'Hash', 'epages']);
		//  | isElement = array.find('Array');
		//  | // returns 1
		this._data.splice(index, 1);
		this.length = this._data.length;
	},

	unique: function () {
		// summary:
		//    returns an (JS) array containing every element only once
		// example:
		//  | array = new epages.lang.Array(['a', 'b', 'c', 'c']);
		//  | uniqueArray = array.unique();
		//  | // returns ['a', 'b', 'c']
		var uniqueData = new epages.lang.Array();
		var i = 0;
		while (i < this._data.length) {
			if (uniqueData.find(this._data[i]) == null) {
				uniqueData.push(this._data[i]);
				i++;
			} else {
				this._data.splice(i,1);
			}
		}
		this.length = this._data.length;
	},

	equals: function(/*array|epages.lang.array*/compareArray) {
		// summary:
		//    returns true, if the given array contains the same element in the same order
		//    than this array, otherwise false.
		//    the parameter can be either a JS array or a epages.lang.Array.
		// example:
		//  | array = new epages.lang.Array(['a', 'b', 'c']);
		//  | return array.equals(['a', 'b', 'c']);
		//  | // returns true
		if (compareArray.length == this.length) {
			if (typeof compareArray._data != "undefined") {
				// compareArray seems to be a epages.lang.Array
				for (var i = 0; i < this.length; i++) {
					if (this._data[i] !== compareArray._data[i]) return false;
				}
			} else {
				// compareArray is a JS array
				for (var i = 0; i < this.length; i++) {
					if (this._data[i] !== compareArray[i]) return false;
				}
			}
			return true; //Boolean
		} else {
			// lengths differ
			return false; //Boolean
		}
	},

	merge: function(/*array|epages.lang.Array*/anotherArray) {
		var c = [];
		if (dojo.isArray(anotherArray)) {
			c = anotherArray;
		} else if(anotherArray._data !== undefined) {
			c = anotherArray._data;
		}
		for (var i = 0; i < c.length; i++) {
			this._data.push(c[i]);
		}
		this.length = this._data.length;
	},

	join: function(delimiter) {
		return this._data.join(delimiter);
	},

	toString: function() {
		var a = this.map(function (v) {
			if (v === undefined)
				return 'undefined';
			else if (v == null)
				return 'null';
			else if (dojo.isArray(v))
				return $A(v).toString();
			else if (dojo.isObject(v))
				return v.toString();
			else
				return '"' + epages.string.escapeJavaScript('' + v) + '"';
		});
		return '[' + a.join(',') + ']';
	},

	sort: function(sortFunc) {
	  return this._data.sort(sortFunc);
	}
};
