/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

	epages.lang.hash $Revision: 1.31.52.2 $

*/
dojo.provide('epages.lang.hash');
dojo.require('epages.lang.array');
dojo.require('epages.string');

$H = function (/*Object*/object) {
	// summary:
	//		returns a new hash, same like new epages.lang.Hash
	// example:
	//	| hash = $H({ key1: 'value1', key2: 'value2'});
	//	| // returns epages.lang.Hash
	return new epages.lang.Hash(object);
};

epages.lang.Hash = /*epages.lang.Hash*/ function (/*Object?*/existingHash) {
	// summary:
	//		returns a new hash, makes a copy of the existing hash
	// example:
	//	|	hash1 = new epages.lang.Hash();
	//	|	// returns an empty epages.lang.Hash
	//	|	hash2 = new epages.lang.Hash({ key1: 'value1', key2: 'value2'});
	// 	|  hash3 = new epages.lang.Hash({
	// 	|		image       : 'test.png',
	// 	|		header      : 'Test',
	// 	|		description : 'long description, long description'
	// 	|	});
	//	|	hash4 = new epages.lang.Hash(hash2);
	//	|	// returns epages.lang.Hash
	this._data = new Object();
	var data = this._data;
	if (existingHash !== undefined) {
		if (typeof existingHash.each == 'function') {
			existingHash.each(function (pair) {
				data[pair.key] = pair.value;
			});
		} else {
			for (var key in existingHash)
				data[key] = existingHash[key];
		}
	}
};

epages.lang.Hash.prototype = {
	_data: undefined,

	data : function () {
		return this._data;
	},

	each : function(iterator) {
		for (var key in this._data) {
			var value = this._data[key];
			var pair = [key, value];
			pair.key = key;
			pair.value = value;
			iterator(pair);
		}
	},

	flatten: function() {
		// summary:
		//		returns xml representation of this hash
		// example:
		// 	|  var hash = new epages.lang.Hash({
		// 	|		image       : 'test.png',
		// 	|		header      : 'Test'
		// 	|	});
		//	|	string = hash.flatten();
		//	|	// returns
		//	|	// <epages.lang.Hash>
		//	|	//  <element><key>image</key><value>test.png</value></element>
		//	|	//  <element><key>header</key><value>Test</value></element>
		//	|	// </epages.lang.Hash>
		var returnvalue = '';
		this.each( function(pair) {
			returnvalue += '<element><key>' + pair.key + '</key><value>' + pair.value + '</value></element>\n';
		});
		return '<epages.lang.Hash>\n' + returnvalue + '</epages.lang.Hash>\n'; //String
	},

	keys: function() {
		// summary:
		//		returns all keys of this hash
		// example:
		// 	|  var hash = new epages.lang.Hash({
		// 	|		image       : 'test.png',
		// 	|		header      : 'Test',
		// 	|		description : 'long description, long description'
		// 	|	});
		//	|	array = hash.keys();
		//	|	// returns ['image','header','description']
		var returnvalue = [];
		this.each( function(pair) {
			returnvalue.push(pair.key);
		});
		return returnvalue; //Array
	},

	values: function() {
		// summary:
		//		returns all values of this hash
		// example:
		// 	|  var hash = new epages.lang.Hash({
		// 	|		image       : 'test.png',
		// 	|		header      : 'Test',
		// 	|		description : 'long description, long description'
		// 	|	});
		//	|	array = hash.keys();
		//	|	// returns ['test.png','Test','long description, long description']
		var returnvalue = [];
		this.each( function(pair) {
			returnvalue.push(pair.value);
		});
		return returnvalue; //Array
	},

	set: function(/*String*/key, /*Object*/value) {
		// summary:
		//		adds a new element to the hash
		// example:
		// 	|  var hash = new epages.lang.Hash({
		// 	|		image       : 'test.png',
		// 	|		description : 'long description, long description'
		// 	|	});
		//	|	hash.set('header','Test');
		//	|	// same as
		// 	|  var hash2 = new epages.lang.Hash({
		// 	|		image       : 'test.png',
		// 	|		description : 'long description, long description',
		// 	|		header      : 'Test'
		// 	|	});
		this._data[key] = value;
	},

	get: function(/*String*/key) {
		// summary:
		//		returns the value of an specified element
		// example:
		// 	|  var hash = new epages.lang.Hash({
		// 	|		image       : 'test.png',
		// 	|		description : 'long description, long description'
		// 	|	});
		//	|	string = hash.get('image');
		//	|	// returns 'test.png'
		return this._data[key];
	},

	remove: function(/*String*/key) {
		// summary:
		//		removes the node with the given key
		// example:
		// 	|  var hash = new epages.lang.Hash({
		// 	|		image       : 'test.png',
		// 	|		description : 'long description, long description'
		// 	|	});
		//	|	bool = hash.remove('image');
		//	|	// returns true
		delete this._data[key];
	},

	existsKey: function(/*String*/key) {
		// summary:
		//		determines, if the given key exists
		// example:
		// 	|  var hash = new epages.lang.Hash({
		// 	|		image       : 'test.png',
		// 	|		description : 'long description, long description'
		// 	|	});
		//	|	bool = hash.existsKey('image');
		//	|	// returns true
		if(this._data[key] !== undefined) return true; //Boolean
		// possible key exists but is undefined
		for (var existingKey in this._data) {
			if (existingKey == key) return true; //Boolean
		}
		return false; //Boolean
	},

	existsValue: function(/*Object*/value) {
		// summary:
		//		returns, if the given value exists
		// example:
		// 	|  var hash = new epages.lang.Hash({
		// 	|		image       : 'test.png',
		// 	|		description : 'long description, long description'
		// 	|	});
		//	|	bool = hash.existsValue('test.png');
		//	|	// returns true
		return this.find(value) !== undefined;
	},

	find: function(/*Object*/value) {
		// summary:
		//		returns the key to the given value or undefined, when not found
		// example:
		// 	|  var hash = new epages.lang.Hash({
		// 	|		a: 'aa',
		// 	|		b: 'bb'
		// 	|	});
		//	|	bool = hash.find('aa');
		//	|	// returns 'a'
		for (var key in this._data) {
			if (this._data[key] === value) return key; //String
		}
		return undefined; //undefined
	},

	toQueryString: function () {
		// summary:
		//		returns uri query string for a hash
		// example:
		//  | var hash = new epages.lang.Hash({ 'schluessel': 'value', 'enc&Schluessel':'enc&Value'});
		//	| var string = hash.toQueryString();
		//	| // returns 'schluessel=value&enc%26Schluessel=enc%26Value'
		var returnvalue = [];
		this.each( function(pair) {
			if (pair.value !== undefined && pair.value != null) {
				var encKey = encodeURIComponent(pair.key);
				if (dojo.isArray(pair.value)) {
					for (var i = 0; i < pair.value.length; i++) {
						returnvalue.push(encKey + '=' + encodeURIComponent(pair.value[i]));
					}
				} else {
					returnvalue.push(encKey + '=' + encodeURIComponent(pair.value));
				}
			}
		});
		return returnvalue.join('&'); //String
	},

	toString: function () {
		// summary:
		//		returns uri query string for a hash
		// example:
		//  | var hash = new epages.lang.Hash({ 'schluessel': 'value', 'enc&Schluessel':'enc&Value'});
		//	| var string = hash.toQueryString();
		//	| // returns 'schluessel=value&enc%26Schluessel=enc%26Value'
		var returnvalue = [];
		this.each( function(pair) {
			var v = pair.value;
			if (v === undefined)
				v = 'undefined';
			else if (v === null)
				v = 'null';
			else if (dojo.isArrayLike(v))
				v = $A(v).toString();
			else if (dojo.isObject(v))
				v = v.toString();
			else
				v = '"' + epages.string.escapeJavaScript('' + v) + '"';

			returnvalue.push('"' + epages.string.escapeJavaScript(pair.key) + '":' + v);
		});
		return '{' + returnvalue.join(',') + '}'; //String
	}
};
