/*
	Copyright (c) 2006-2007, epages Software GmbH
	All Rights Reserved.

	epages.uri.* $Revision: 1.11.4.3 $

*/
dojo.provide("epages.lang.uri");
dojo.require("epages.lang.hash");

epages.lang.uri = {

	parts: function(uri) {
		// summary:
		//		returns an object with the parts of an uri
		// description:
		//		available parts are: protocol, host, path, query, parameters (Object), anchor
		// example:
		// 	|	var parts = epages.lang.uri.parts('http://www.host.com/path/to/somewhere?action=go#top');
		//	|	parts == {
		//	|		protocol: 'http',
		//	|		host: 'www.host.com',
		//	|		path: '/path/to/somewhere',
		//	|		query: '?action=go',
		// 	|		anchor: '#top',
		//	|		parameters: {
		//	|			action: 'go'
		//	|		}
		// 	|	}
		var parts = {};
		var regExpResult = uri.match(/^((?:(.+)\:\/\/([^\/]+))?((?:\/|[^\?#]).*?)?(\?.*?)?(#.*)?)$/);
		if (regExpResult) {
			parts = {
				'href':				regExpResult[1],
				'protocol': 	regExpResult[2],
				'host': 			regExpResult[3],
				'path': 			regExpResult[4],
				'query': 			regExpResult[5],
				'anchor':			regExpResult[6],
				'parameters': {}
			};
			parts.parameters = this.splitQueryString(parts.query);
		}
		return parts;
	},

	splitQueryString: function(uri) {
		// summary:
		//		returns an object containing the elements of a query string
		// example:
		// 	|	var parts = epages.lang.uri.parts('?action=go');
		//	|	parts == {
		//	|		action: 'go'
		// 	|	}
		var returnValue = {};
		var urlSplits = (new RegExp(/\??(.*?)(#.*)?$/g)).exec(uri);
		if (urlSplits) {
			var queryString = urlSplits[1];
			var splits = queryString.split('&');
			for (var i = 0; i < splits.length; i++) {
				var querySplits = (new RegExp(/(.+)\=(.*)/)).exec(splits[i]);
				if (querySplits) {
					returnValue[decodeURIComponent(querySplits[1])] = decodeURIComponent(querySplits[2]);
				}
			}
		}
		return returnValue;
	},

	toQueryString: function(/*Object*/data) {
		// summary:
		//	Generates a query string from a data structure.
		// description:
		//	Needs a hash (hashlike) or an array[string[2]]. The first element
		//	of each sub array will be used as key, the second as the value.
		// example:
		//	| r = epages.lang.uri.toQueryString( { test: 123, test2: abc} );
		//	| r = epages.lang.uri.toQueryString( [ ['test', '123'], ['test2', 'abc'] ] );
		//	| // r (in both cases) == 'test=123&test2=abc'
		if (dojo.isArray(data)) {
			// expects strict array structure
			var returnValue = [];
			for (var i = 0; i < data.length; i++) {
				if (!dojo.isArray(data[i])) {
					console.debug('data element is not an array: ' + (typeof data[i]));
					continue;
				}
				returnValue.push(encodeURIComponent(data[i][0]) + '=' + encodeURIComponent(data[i][1]));
			}
			return returnValue.join('&');
		} else if (dojo.isObject(data)) {
			// use the function provided by a hash
			return $H(data).toQueryString();
		}
	},

	fromParts: function(parts) {
		// summary:
		//		returns the uri for an object of its parts
		var result = '';
		if (parts.protocol) {
			result += parts.protocol + '://';
		}
		if (parts.host) {
			result += parts.host;
		}
		if (parts.path) {
			result += parts.path;
		}
		if (dojo.isObject(parts.parameters)) {
			var qs = this.toQueryString(parts.parameters);
			if (qs) result += '?' + qs
		} else if (parts.query) {
			result += parts.query;
		}
		if (parts.anchor) {
			result += parts.anchor;
		}
		return result;
	},

	isAbsolute: function(/*String*/uri) {
		// summary: uri starts with scheme or a slash
		var rx = new RegExp(/^((http|https|ftp)\:\/\/|\/)/g);
		return rx.test(uri);
	},

	getRelative: function(/*String*/baseUrl, /*String*/url) {
		var baseUrlParts = epages.lang.uri.parts(baseUrl);
		var urlParts = epages.lang.uri.parts(url);

		// protocols or hosts differ
		if ((urlParts.protocol != null && urlParts.protocol != baseUrlParts.protocol) ||
				(urlParts.host != null && urlParts.host != baseUrlParts.host)) return url;

		baseUrlParts.path = baseUrlParts.path.replace(/\/+$/, '');
		urlParts.path = urlParts.path.replace(/\/+$/, '');
		var basePathSplitted = baseUrlParts.path.split('/');
		var urlPathSplitted = urlParts.path.split('/');

		// remove equal parts
		while (basePathSplitted.length > 0) {
			var basePathPart = basePathSplitted.shift();
			var urlPathPart = urlPathSplitted.shift();
			if (urlPathPart == null) {
				basePathSplitted.unshift(basePathPart);
				break;
			} else if (basePathPart != urlPathPart) {
				basePathSplitted.unshift(basePathPart);
				urlPathSplitted.unshift(urlPathPart);
				break;
			}
		}
		var result = '';
		for (var i = 0; i < basePathSplitted.length; i++) { result += '../'; }
		return result + urlPathSplitted.join('/');
	}

};
