﻿use Test::More tests => 8;
use strict;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw( ExistsObjectByPath LoadObjectByPath );
use DE_EPAGES::WebInterface::API::FakeRequest;
use DE_EPAGES::Core::API::File qw (GetFileContent GetFileMD5Digest ExistsFile DeleteFile fsplit);
use HTTP::Cookies;
use HTTP::Request;
use LWP::UserAgent;
use HTTP::Headers;

my $SHOPPATH = "/Shops/DemoShop";
my $TESTIMAGEFILENAME = 'test.jpg';

sub LoginToShop {
    my $Shop = shift;

    # login into demoshop mbo
    my $URL = $Shop->get('WebServerScriptAdminUrl');
    my $Browser = LWP::UserAgent->new;
    # $Browser->proxy(['http'], 'http://127.0.0.1:8888/');    # fiddler

    $Browser->cookie_jar({});
    my $LoginResponse = $Browser->post(
        $URL,
        [
            'ChangeAction' => 'SaveLoginForm',
            'Login' => 'admin',
            'Password' => 'admin',
            'ViewAction' => 'MBO-ViewMBO'
        ]
    );
    my $LoginCookies = HTTP::Cookies->new;
    $LoginCookies->extract_cookies( $LoginResponse );
    cmp_ok( $LoginResponse->code, '==', 200, 'Login 200 OK' );

    $URL = $LoginResponse->header('content-base');

    return ($URL, $Browser);
}

sub UploadMultiPartFile {
    my ($Object, $SessionURL, $Browser, $ChangeAction, $UseMediaGallery) = @_;
    $UseMediaGallery = $UseMediaGallery ? 1 : 0;
    my $TestDir = 'testdirectory';

    my $Headers = HTTP::Headers->new;
    $Headers->header(
        'handleashtml' => 1,
        'Content-Type' => 'multipart/form-data',
        'Accept' => 'text/html'
    );

    $SessionURL .= '?ViewAction=JSONViewResponse&ChangeAction='.$ChangeAction.'&UseMediaGallery='.$UseMediaGallery.'&ObjectID='.$Object->id;
    my $TestFileContent = GetFileContent($TESTIMAGEFILENAME, { 'binary' => 1 });

    my $UploadResponse = $Browser->post(
        $SessionURL,
        'handleashtml' => 1,
        'Content-Type' => 'multipart/form-data',
        'Content' => [
            'method' => 'submit',
            'Name' => 'TestUpload',
            'Directory' => $TestDir,
            'NewFile' => [ $TESTIMAGEFILENAME ]
        ]
    );

    # send
    cmp_ok( $UploadResponse->code, '==', 200, '200 OK' );

    # compare form binary data with file binary
    my $UploadDir;
    if($UseMediaGallery) {
        my $MediaGallery = $Object->getSite()->get('MediaGallery');
        $UploadDir = $MediaGallery->absolutePath('');
        ok(ExistsFile(join('/', $UploadDir, $TestDir, $TESTIMAGEFILENAME)), 'upload successful');
        my (undef, $MgtFileName, $MgtExt) = fsplit($UploadDir.'/'.$TESTIMAGEFILENAME);
        my $MgtFile = $MgtFileName.',mgt'.$MgtExt;
        ok(ExistsFile(join('/', $UploadDir, $TestDir, $MgtFile)), 'mgt conversion successful');
        CheckFile($TESTIMAGEFILENAME, join('/', $UploadDir, $TestDir, $TESTIMAGEFILENAME));
        DeleteFile(join('/', $UploadDir, $TestDir, $TESTIMAGEFILENAME));
        DeleteFile(join('/', $UploadDir, $TestDir, $MgtFile)) if ExistsFile(join('/', $UploadDir, $TestDir, $MgtFile));
    } else {
        $UploadDir = $Object->get('PublicPath');
        ok(ExistsFile(join('/', $UploadDir, $TestDir, $TESTIMAGEFILENAME)), 'upload successful');
        CheckFile($TESTIMAGEFILENAME, join('/', $UploadDir, $TestDir, $TESTIMAGEFILENAME));
        DeleteFile(join('/', $UploadDir, $TestDir, $TESTIMAGEFILENAME));
    }
}

sub UploadOctetFile {
    my ($Object, $SessionURL, $Browser, $ChangeAction, $UseMediaGallery) = @_;
    $UseMediaGallery = $UseMediaGallery ? 1 : 0;

    my $Headers = HTTP::Headers->new;
    $Headers->header(
        'handleashtml' => 1,
        'content-disposition' => 'attachement; filename="'.$TESTIMAGEFILENAME.'";',
        'Content-Type' => 'application/octet-stream',
        'Accept' => 'text/html'
    );

    $SessionURL .= '?ViewAction=JSONViewResponse&ChangeAction='.$ChangeAction.'&UseMediaGallery='.$UseMediaGallery.'&ObjectID='.$Object->id;
    my $TestFileContent = GetFileContent($TESTIMAGEFILENAME, { 'binary' => 1 });
    my $UploadRequest = HTTP::Request->new(
        'POST',
        $SessionURL,
        $Headers,
        ${$TestFileContent}
    );
    my $UploadResponse = $Browser->request($UploadRequest);

    # send
    cmp_ok( $UploadResponse->code, '==', 200, '200 OK' );

    # compare form binary data with file binary
    my $UploadDir;
    if($UseMediaGallery) {
        my $MediaGallery = $Object->getSite()->get('MediaGallery');
        $UploadDir = $MediaGallery->absolutePath('');
        ok(ExistsFile($UploadDir.'/'.$TESTIMAGEFILENAME), 'upload successful');
        my (undef, $MgtFileName, $MgtExt) = fsplit($UploadDir.'/'.$TESTIMAGEFILENAME);
        my $MgtFile = $MgtFileName.',mgt'.$MgtExt;
        ok(ExistsFile($UploadDir.'/'.$MgtFile), 'mgt conversion successful');
        CheckFile($TESTIMAGEFILENAME, $UploadDir.'/'.$TESTIMAGEFILENAME);
        DeleteFile($UploadDir.'/'.$TESTIMAGEFILENAME);
        DeleteFile($UploadDir.'/'.$MgtFile) if ExistsFile($UploadDir.'/'.$MgtFile);
    } else {
        $UploadDir = $Object->get('PublicPath');
        ok(ExistsFile($UploadDir.'/'.$TESTIMAGEFILENAME), 'upload successful');
        CheckFile($TESTIMAGEFILENAME, $UploadDir.'/'.$TESTIMAGEFILENAME);
        DeleteFile($UploadDir.'/'.$TESTIMAGEFILENAME);
    }
}

sub CheckFile() {
    my ($Source, $Target) = @_;
    cmp_ok(
        GetFileMD5Digest($Source),
        'eq',
        GetFileMD5Digest($Target),
        'file content md5 hash matches'
    );
}

sub TestUploads {
    my $Shop = shift;

    my ($SessionURL, $Browser) = LoginToShop($Shop);
    UploadOctetFile($Shop, $SessionURL, $Browser, 'UploadFiles', 0);
    UploadMultiPartFile($Shop, $SessionURL, $Browser, 'UploadFiles', 0);
}

sub Main {
    if( ExistsObjectByPath($SHOPPATH) ) {
        my $Shop = LoadObjectByPath($SHOPPATH);
        ok( $Shop->id, "load demoshop " . $Shop->id );
        TestUploads($Shop);
    } else {
        SKIP: {
            skip "no DemoShop installed", 8;
        };
    }
    return;
}

RunScript( Message => 'long', Sub=> sub {
    RunOnStore( Store => GetTestSite(), Sub => \&Main );
});