use strict;
use constant TESTS => 27;
use Test::More tests => TESTS;
use DE_EPAGES::Cartridge::API::Table::Cartridge qw( ExistsCartridgeByPackage );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Warning qw( BlockWarnings );
use DE_EPAGES::Database::API::Connection qw( RunOnStore GetTestSite );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath ExistsObjectByPath LoadRootObject);
use DE_EPAGES::Object::API::Language;
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::Product::API::Basket qw ( AddProducts );

my $ROOTPATH = '/Test';

sub TestBasket {

    my $Shop = LoadObjectByPath('/Test/TestShop');
    my $User = $Shop->path('Users/TestUser');
    $DE_EPAGES::Object::API::Language::LANGUAGEID = $User->get('LanguageID');
    my $Product = $Shop->path('Products/Shirt/SubProducts/"Shirt blue"');
    my $aBaskets;
    my $aWarnings = BlockWarnings( sub {
        $aBaskets = AddProducts($User, {}, [{
            'Product'  => $Product,
            'Quantity' => 12,
        }]);
    });
    foreach my $Warning (@$aWarnings) {
        print STDERR "Warning: " . $Warning->code . ": " . $Warning->message . "\n";
    }
    ok( 0 == scalar @$aWarnings, "no warnings in AddProducts()" );
    my $Basket = $aBaskets->[0];
    my $Container = $Basket->get('LineItemContainer');
    is( $User->get('CurrencyID'), 'EUR', 'this test case only checks prices in EUR' );
    is( $Container->get('LocaleID'), $User->get('LocaleID'), 'Container.LocaleID' );
    is( $Container->get('CurrencyID'), $User->get('CurrencyID'), 'Container.CurrencyID' );
    ok( $Container->get('TaxArea')->alias eq 'EU', 'TaxArea' );
    ok( $Container->get('TaxModel') == 1, 'TaxModel' );

    ok( 20.0 == $Container->get('Shipping')->get('LineItemPrice'), "Shipping.LineItemPrice" );
    ok( 0.19 == $Container->get('Shipping')->get('TaxRate'), "Shipping.TaxRate" );
    ok( 20.0 == $Container->get('Payment')->get('LineItemPrice'), "Payment.LineItemPrice" );
    ok( 0.19 == $Container->get('Payment')->get('TaxRate'), "Payment.TaxRate" );

    # actually there is only one line item
    foreach my $L ( @{$Container->get('LineItems')} ) {
        ok( 12 == $L->get('Quantity'), "LineItemProduct.Quantity" );
        ok( 6.99 == $L->get('BasePrice'), "LineItemProduct.BasePrice" );
        ok( 83.88 == $L->get('LineItemPrice'), "LineItemProduct.LineItemPrice" );
        ok( 0 == $L->get('TaxAmount'), "LineItemProduct.TaxAmount" );
    }

    is( $Container->get('TotalBeforeTax'), 113.89, 'Container.TotalBeforeTax' );
    is( $Container->get('GrandTotal'), 123.88, 'Container.GrandTotal' );
    is( $Container->get('TotalTax'), 9.99, 'Container.TotalTax' );

    $Basket->delete;
    return;
}

sub TestDownloadBasket {

    my $Shop = LoadObjectByPath('/Test/TestShop');
    my $User = $Shop->path('Users/TestUser');
    $DE_EPAGES::Object::API::Language::LANGUAGEID = $User->get('LanguageID');
    my $Product = $Shop->path('Products/DownloadProduct/SubProducts/DownloadProduct-de');
    my $aBaskets;
    my $aWarnings = BlockWarnings( sub {
        $aBaskets = AddProducts($User, {}, [{
            'Product'  => $Product,
            'Quantity' => 12,
        }]);
    });
    foreach my $Warning (@$aWarnings) {
        print STDERR "Warning: " . $Warning->code . ": " . $Warning->message . "\n";
    }
    ok( 0 == scalar @$aWarnings, "no warnings in AddProducts()" );
    my $Basket = $aBaskets->[0];
    my $Container = $Basket->get('LineItemContainer');
    is( $User->get('CurrencyID'), 'EUR', 'this test case only checks prices in EUR' );
    is( $Container->get('CurrencyID'), $User->get('CurrencyID'), 'Container.CurrencyID' );

    my $aLineItems = $Container->get('LineItems');
    is( scalar @$aLineItems, 1, "number of products in basket" );

    my $LineItem = $aLineItems->[0];
    ok( !$LineItem->get('IsLineItemProductBundle'), "no bundle lineitem" );
    ok( $LineItem->get('IsLineItemDownloadProduct'), "LineItemProduct.IsLineItemDownloadProduct" );

    my $MaxCount = $Product->getWithParent('MaxDownloadCount');
    is( $MaxCount, 3, "Product.MaxDownloadCount" );

    my %MaxDownloadCounts = map { $_->get('DownloadProductMapID') => $MaxCount }
        @{ $Product->get('DownloadProductMaps') };
    $LineItem->set({'MaxDownloadCounts' => \%MaxDownloadCounts});
    like( $LineItem->get('MaxDownloadCountString'), qr/\d+=3;\d+=3;\d+=3;\d+=3/, 'LineItemProduct.MaxDownloadCountString' );

    $Basket->delete;
    return;
}

sub Main {

    RunOnStore( Store => GetTestSite(), Sub => sub {
        SKIP: {
            if( ! ExistsCartridgeByPackage('DE_EPAGES::ShippingTypes' ) ) {
                skip( "DE_EPAGES::ShippingTypes or is not installed", TESTS );
            }
            if( ! ExistsCartridgeByPackage('DE_EPAGES::PaymentTypes' ) ) {
                skip( "DE_EPAGES::PaymentTypes or is not installed", TESTS );
            }

            ok( Init(), 'initialization' );

            TestBasket();
            TestDownloadBasket();

            ok( Cleanup(), 'Cleanup' );
        }
    });
}

sub Cleanup {
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    Cleanup();
    LoadRootObject()->insertChild('Object', {'Alias'=>'Test'});
    ImportObjects('import.xml', $ROOTPATH);
    ImportObjects('initial.xml', $ROOTPATH);
    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);
