#!/bin/env perl.exe

use strict;
use Socket;
use DE_EPAGES::Core::API::Error qw (Error ExistsError GetError);
use DE_EPAGES::TLE::API::Template qw (ProcessFileTemplate);
use DE_EPAGES::WebInterface::API::Config qw (GetRequestRouterMonitors);
use DE_EPAGES::WebInterface::API::RequestRouterMonitor;
use DE_EPAGES::Core::API::Script qw ( RunScript );

my $BaseDir = $0;
$BaseDir =~ s/[\\\/][^\\\/]+$//;
my $TemplateFile = $BaseDir . '/server.tmpl';
my %TEMPLATE_REQUEST = map { ($_ => undef) } qw( server debug rr pool );

#========================================================================================
# function     Parse
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = Parse();
#----------------------------------------------------------------------------------------
# description  Parses web form data of of the CGI script.
#               Joins multiple value of the same name with \0 characters.
# return       $hValues | form values | ref.hash.string
#========================================================================================
sub Parse {
    # Get the input
    my $buffer = $ENV{QUERY_STRING};
    my %FORM;
    return \%FORM unless defined $buffer;

    if( $ENV{'REQUEST_METHOD'} eq 'POST' ) {
        my $postData;
        read STDIN, $postData, $ENV{'CONTENT_LENGTH'};
        $buffer = $postData.'&'.$buffer;
    }

    # Split the name-value pairs
    my @pairs = split(/&/, $buffer);
    my $pair;
    foreach $pair (@pairs) {
        my($name, $value) = split(/=/, $pair);
        $value =~ tr/+/ /;
        $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
        die "duplicate value for $name" if exists $FORM{$name};
        $FORM{$name} = $value;
    }

    return \%FORM;
}

#--------------------------------------------------------------------------
# function main
#--------------------------------------------------------------------------

sub SortLoop {
    my ($aLoop, $OrderBy, $Numeric, $Desc) = @_;
    return [
        sort {
            $Numeric
            ?
                $Desc
                ? $b->{$OrderBy} <=> $a->{$OrderBy}
                : $a->{$OrderBy} <=> $b->{$OrderBy}
            :
                $Desc
                ? $b->{$OrderBy} cmp $a->{$OrderBy}
                : $a->{$OrderBy} cmp $b->{$OrderBy}
        }
        @$aLoop
    ]
}

sub main {

    my $Form = Parse();

    my @Servers = defined $Form->{'Server'} ? split /\0/, $Form->{'Server'} : ();
    if (not scalar @Servers) {
        my $aMCs = GetRequestRouterMonitors();
        @Servers = map { $_->[0].':'.$_->[1] } @$aMCs;
    }

    my $Template = $Form->{'Template'};
    $Template = 'server' unless defined $Template;
    $Template = 'server' if $Template eq 'sas60';
    $Template = 'debug'  if $Template eq 'sas';
    $Template = 'server' unless exists $TEMPLATE_REQUEST{$Template};

    my $OrderBy = $Form->{OrderBy};
    my $Numeric = $Form->{Numeric} || 0;
    my $Desc = $Form->{Desc} || 0;

    my @ServerLoop;
    foreach my $Server ( @Servers ) {

        if( $Server !~ /(.+)\:(.+)/ ) {
            die "Invalid server format: $Server";
        }

        my $ServerName  = $1;
        my $MonitorPort = $2;

        my $hVars = {};
        eval {
            my $Monitor = DE_EPAGES::WebInterface::API::RequestRouterMonitor->new(
                Address => $ServerName,
                Port => $MonitorPort
            );

            if($Form->{'resetStatisticsForAllPools'}) {
                $Monitor->resetStatisticsForAllPools;
            }
            elsif($Form->{'resetStatisticsForPool'}) {
                my $PoolName = $Form->{'resetStatisticsForPool'};
                $Monitor->resetStatisticsForPool($PoolName);
            }

            if( $Template eq 'server' ) {
                $hVars->{'ApplicationServers'} = SortLoop( $Monitor->servers,
                    $OrderBy, $Numeric, $Desc );
            }
            elsif( $Template eq 'debug' ) {
                $hVars->{'ApplicationServers'} = SortLoop( $Monitor->debug,
                    $OrderBy, $Numeric, $Desc );
            }
            elsif( $Template eq 'rr' ) {
                $hVars->{'RequestRouters'} = SortLoop( $Monitor->requestRouters,
                    $OrderBy, $Numeric, $Desc );
            }
            elsif( $Template eq 'pool' ) {
                $hVars->{'Pools'} = SortLoop( $Monitor->pools,
                    $OrderBy, $Numeric, $Desc );
            }

            $hVars->{'ConnectionFailed'}  = 0;
        };
        if (ExistsError()) {
            my $Error = GetError();
            $hVars->{'ConnectionFailed'}  = 1;
            $hVars->{'FailedString'}      = $Error->shortMessage;
        }
        $hVars->{'ServerName'}  = $ServerName;
        $hVars->{'MonitorPort'} = $MonitorPort;
        $hVars->{'Server'} = $ServerName.':'.$MonitorPort;
        push @ServerLoop, $hVars;
    }
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime( time );

    my %Vars = (
        'Template'      => $Template,
        'OrderBy'       => $OrderBy,
        'Numeric'       => $Numeric,
        'Desc'          => $Desc,
        'Servers'       => \@ServerLoop,
        'CurrentTime'   => sprintf( "%02d:%02d:%02d", $hour, $min, $sec ),
        'CurrentDate'   => sprintf( "%02d-%02d-%02d", 1900+$year, $mon+1, $mday ),
        );

    %Vars = (%Vars, %ENV);

    if( exists $Form->{'Template'} ) {
        $TemplateFile = $BaseDir .'/'. $Form->{'Template'} .'.tmpl';
    }

    print ProcessFileTemplate( $TemplateFile, \%Vars );
    chmod(oct(770), glob("$BaseDir/*.ctmpl"));

    return;
}

eval {

    RunScript(
        'Sub' => \&main
    );

};
if( $@ ) {
    print "Content-Type: text/html\n\n";
    print "<HTML><HEAD><TITLE>ScriptError</TITLE></HEAD>\n";
    print "<H1>Script Error</H1>";
    print "<PRE>$@</PRE>\n";
    print "</BODY></HTML>\n";
}

