#!/bin/env bash

PWD=`pwd`

usage() {
  echo "Usage:" 1>&2
  echo "  EPAGES_PROJECT=epages6 $0 -i INITIAL_VERSION -p PATCH_VERSION -b BACKUP_DIR -P EPAGES_PROJECT [-R]" 1>&2
  echo "Options:" 1>&2
  echo "   EPAGES_PROJECT: envvar of the epages project, usually epages6" 1>&2
  echo "   -i: initial version (integer < 10^19)" 1>&2
  echo "   -p: patch version (dotted)" 1>&2
  echo "   -b: backup directory" 1>&2
  echo "   -R: restore" 1>&2
  exit 1
}

unset INITIAL_VERSION NEW_PATCH_VERSION BACKUP_DIR RESTORE EPAGES_PROJECT

while getopts i:p:b:RP: c ; do
  case $c in
    i) INITIAL_VERSION=$OPTARG ;;
    p) NEW_PATCH_VERSION=$OPTARG ;;
    b) BACKUP_DIR=$OPTARG ;;
    R) RESTORE=$c ;;
    P) EPAGES_PROJECT=$OPTARG ;;
    *) echo "$0: Unknown option: $c" ; usage ;;
  esac
done
shift `expr $OPTIND - 1`

if [ ! -w /etc/passwd ] ; then
  echo "$0: only 'root' can execute this script." 1>&2
  exit 1
fi

[ -z "$EPAGES_PROJECT" ] && { echo "$0: Option required: -P" ; usage; }
[ -z "$NEW_PATCH_VERSION" ] && { echo "$0: Option required: -p" ; usage; }
[ -z "$INITIAL_VERSION" ] && { echo "$0: Option required: -i" ; usage; }
[ -n "${INITIAL_VERSION//[0-9]/}" ] && { echo "$0: Not a number: -i $INITIAL_VERSION" ; usage; }
INITIAL_VERSION=`expr $INITIAL_VERSION + 0 2>/dev/null`
[ ${#INITIAL_VERSION} -gt 18 ] && { echo "$0: Number must no exeed 10^19: -i $INITIAL_VERSION" ; usage; }

# set environment

export EPAGES_PROJECT
EP_DEFAULT=/etc/default/"$EPAGES_PROJECT"

if grep -q "^PATCH_VERSION=$NEW_PATCH_VERSION$" "$EP_DEFAULT" ; then
  [ -z "$RESTORE" ] && {
    echo "INFO: $BN skipped, because '$EP_DEFAULT' already patched" 1>&2
    exit 0
  }
elif [ -n "$RESTORE" ] ; then
  echo "INFO: $BN skipped, because '$EP_DEFAULT' not patched" 1>&2
  exit 0
fi

[ -n "$BACKUP_DIR" -a ! -d "$BACKUP_DIR" ] && { mkdir -p "$BACKUP_DIR" || exit 1; }

. "$EP_DEFAULT"

EPAGES_ETC=$(dirname "$EPAGES")/etc

# restore epages scripts

if [ -n "$RESTORE" ] ; then
  if [ -n "$BACKUP_DIR" ] ; then
     for i in default/"$EPAGES_PROJECT" init.d/"$EPAGES_PROJECT" init.d/epages6su ; do
      [ -f "$BACKUP_DIR"/$i ] || continue
      cat "$BACKUP_DIR"/$i > /etc/$i
    done
    /etc/init.d/"$EPAGES_PROJECT" perm admin
  fi
  exit 0
fi

# backup epages scripts

if [ -n "$BACKUP_DIR" ] ; then
  for i in default/"$EPAGES_PROJECT" init.d/"$EPAGES_PROJECT" init.d/epages6su ; do
    [ -f /etc/$i ] || continue
    mkdir -p -m 755 "$BACKUP_DIR"/${i%/*}
    cat /etc/$i > "$BACKUP_DIR"/$i
  done
fi

# edit init.d/epages6

cd "$EPAGES_ETC" || exit 1

echo "$EPAGES_PROJECT" > epages-project
sed "s,_EPAGES_PROJECT_,$EPAGES_PROJECT,g" init.d/epages6.ept > init.d/epages6
chmod 755 init.d/epages6

# edit default/epages6

if [ $INITIAL_VERSION -lt 6010000000 ] ; then

  V1="AMUSER BS_SERVER CHECK_NFS DSLISTEN DSQUERY EPAGES EPAGES_ACL EPAGES_APPUSER EPAGES_ASPOOLDBCACHE EPAGES_ENABLE_IPv6"
  V1="$V1 EPAGES_IGNORE_DEMOSHOP EPAGES_IGNORE_SITE EPAGES_IGNORE_CRON EPAGES_IGNORE_APPSRV EPAGES_IGNORE_REQSRV"
  V1="$V1 EPAGES_IGNORE_SYBSRV EPAGES_IGNORE_WEBSRV EPAGES_MYSQL_CONNECTION EPAGES_PROJECT EPAGES_TZ EPAGES_WEBGROUP"
  V1="$V1 EPAGES_WEBUSER HTTPD_OPTS IP_BINDING LOCAL_IPS MAX_NOFD MS_SERVER SERVER SHUTDOWN_DB_SERVER SYBVER"

  V2="EPAGES_CARTRIDGES EPSERVICE EP_RR EPAGES_PERL PERL EPAGES_SHARED EPAGES_CONFIG EPAGES_LOG EPAGES_STATIC EPAGES_STORES"
  V2="$V2 EPAGES_WEBROOT HTTPD_ROOT HTTPD__SV HTTPD_PID EPAGES_REWRITE_CONF SYBASE SYBROOT SCROOT SYBASE_JRE"
  V2="$V2 SYBASE_ASE_LOG RUNDB STARTDB SYSAM_BIN SYBASE_JRE6_32 SYBASE_JRE6_64 SYBASE_JRE6 EPAGES_JAVA"

  if [ -f "$EP_DEFAULT".local ] ; then
    V0=$( unset $V1
      . "$EP_DEFAULT".local
      for i in $V1 ; do
        # check if variable is defined in "$EP_DEFAULT".local
        [[ $(eval echo \${!$i[\*]}) = 0 ]] || echo $i
      done
    )
  else
    V0=$V1
  fi

  cp -p default/epages6.ept default/epages6

  INI_CONFIG="$EPAGES"/bin/ini-config.sh

  for i in $V0 $V2 ; do
    DEFAULT=$("$INI_CONFIG" -f default/epages6 -g -k $i 2>/dev/null)
    DEFAULT=$(eval echo $DEFAULT)
    ACTUAL=$(eval echo '$'$i)
    [[ "$DEFAULT" = "$ACTUAL" ]] && continue
    echo "$ACTUAL" | grep -q '[/|&;()<>[:space:]]' && ACTUAL="\"$ACTUAL\""
    "$INI_CONFIG" -f default/epages6 -k $i -v "$ACTUAL" -i
  done

  if [ -d "$SYBASE/shared/sybcentral600/jre" ] ; then
    EPAGES_JAVA='"$SYBASE/shared/sybcentral600/jre"'
    [[ "$SCROOT" = "$SYBASE/shared/sybcentral600" ]] && EPAGES_JAVA='"$SCROOT/jre"'
    [[ "$SYBASE_JRE" = "$SCROOT/jre" ]] && EPAGES_JAVA='"$SYBASE_JRE"'
    "$INI_CONFIG" -f default/epages6 -k EPAGES_JAVA -v $EPAGES_JAVA -i
  fi

  echo "PATCH_VERSION=$NEW_PATCH_VERSION" >> default/epages6

fi

# link epages scripts

for i in default/"$EPAGES_PROJECT" init.d/"$EPAGES_PROJECT" init.d/epages6su ; do
  [ -f /etc/$i ] || continue
  rm -f /etc/$i
  if [ ${i#*/} = epages6su ] ; then
    ln -s "$EPAGES_ETC"/$i /etc/$i
  else
    ln -s "$EPAGES_ETC"/${i%/*}/epages6 /etc/$i
  fi
done

# set access rights

chmod 755 /etc/default/"$EPAGES_PROJECT"
chmod 740 /etc/init.d/"$EPAGES_PROJECT"
/etc/init.d/"$EPAGES_PROJECT" perm admin
